/*	Module to create ccf functions for amplitude modulated tones.
 * 	(BC 3 months)	TOR- created. (BC = before comments)
 *	(8-23-88)	TOR- commented somewhat.
 *			Things to work on: find reason that fudge scale
 *			factors need to be used to make model work out.
 *	(5-26-89)	TOR - adapted for NeXT.
 *
 */

#import <stdio.h>
#import <math.h>
#import <stdlib.h>
#import "BiDis.h"
#import "Nerve.h"
#import "ccf.h"

#define MAXFACT		1.0/NREALPTS
#define SCALE		39.06	/* magic number to keep results in line with
					noise results. */
#define AMFUDGE		1e3	/* need to keep numbers from underflowing
					in ffts */
#define CONSTCROSS	61.03 /* (50 * TStep * AMFUDGE)^2 * SCALE */

float LPFSq[NREALPTS + 1];	/* Gsqd filter for crosscorellation function*/
float LPF[NREALPTS + 1];		/* G filter for signal */

extern double 	threshold(double f),
				Hm(double f,double fm),
				G(double f),
				GSqd(double f),
				Sqrt(double x);
extern Fraction	AMtoneActive(double powL, double powR, double IID, double fm);
extern int 	G_HasChanged(),
			isinregion(double f, double fm, Fraction fraction);
extern 	fft(float *b,int p,int q,int d),
		multfft(int n,float *m1,float *m2,float *p),
		AMRectify(float *x, int n);
extern float costb[];	/* table lookup for speed */
extern Fraction *activeTable;

/* function takes the 3 coefficients of the three cosine waves and
 	returns the Mean squared value in db.
 */
double findMS(float *A)
{
	double MS;
	int n;

	MS = 0.0;
	for (n=0; n<3; n++) {
		MS += A[n]*A[n];
	}
	
	if (MS != 0.0)
		MS = 10.0 * log10(.5 * MS);
	
	return MS;
}

/* takes frequency and TSTEP and returns nearest frequency value that can
  	be used in fft arrays to eliminate calculation errors.
 */
double fixfreq(double f, double ts)		/* Hz, msec */
{
	double f0 = 1000.0 / (ts*NREALPTS);
	return (double)(((int)(f/f0 + .5)) * f0);
}


/* note: these do not have factor of two */
void LPFinit(double newtstep)	/* msec */
{
	static double 	curtstep = -1.0;
	register int f;
	double freq, curfstep;

	if ( newtstep == curtstep && !G_HasChanged() )
		return;
	curtstep = newtstep;
	curfstep =  (1000./(NREALPTS*curtstep));	/* Hz */
	for (f = 0, freq = 0.0; f < NREALPTS; f+=2, freq += curfstep) {
		LPF[f] = G(freq);
		LPF[f+1] = 0.0;
	}
	LPF[1] = G(freq);
}

void LPFSqinit(double newtstep)		/* msec */
{
	register int f;
	static double curSqtstep = -1.0;
	double freq, curfstep;

	if ((newtstep == curSqtstep) && !G_HasChanged() )
		return;
	curSqtstep = newtstep;
	curfstep =  (1000./(NREALPTS*curSqtstep));	/* Hz */
	for (f = 0, freq = 0.0; f < NREALPTS; f+=2, freq += curfstep) {
		LPFSq[f] = GSqd(freq); /* Factor of 2 for multfft() */
		LPFSq[f+1] = 0.0;
	}
	LPFSq[1] = GSqd(freq);
}


/* This procedure actually finds the cross-correlation function and
	scales it appropriately.
*/
void DoCrossCor(ccf *Lm, int f, float *x1, float *x2)
{
	int n;
	double fudge = 1.0;

	LPFSqinit(DISP_TSTEP);
	
	/* must divide by 2 for fft */
	for(n = 0; n < NREALPTS; n++) {
		x2[n] = x2[n] / 2;
		x1[n] = x1[n] / 2;
	}
	fft(x2, NCOMPPTS, LOGNCOMPPTS, 1);
	fft(x1, NCOMPPTS, LOGNCOMPPTS, 1);
	for(n = 3; n < NREALPTS; n += 2) {
		x1[n] = -x1[n];
	}
	multfft(NCOMPPTS, x2, x1, x1);
	multfft(NCOMPPTS, x1, LPFSq, x1);
	fft(x1, NCOMPPTS, LOGNCOMPPTS, 0);
	
	/* MAXFACT divides by NREALPTS for invfft */
	fudge = MAXFACT * SCALE;
	
	for(n = 1; n <= DISP_TUP; n++) {
		Lm->R[f][-n] = (double)x1[NREALPTS - n] * fudge;
		Lm->R[f][n] = (double)x1[n] * fudge;
	}
	Lm->R[f][0] = x1[0] * fudge;
}


/* given the amplitude of the signal, this procedure Hm filters and
   returns the coefficients of the three waves.
 */
void findCosCoef(double fcm, double fm, double fc, float *A,
				 double SPL,double modp)
{
	A[0] = Sqrt(2.0 * pow(10.0,(SPL / 10)) / (1.0 + modp * modp / 2.0));
	A[2] = A[1] = A[0] * modp / 2.0;
	A[0] *= Hm(fc,fcm);
	A[1] *= Hm(fc + fm, fcm);
	A[2] *= Hm(fc - fm, fcm);
}

/* returns signal in x.  the  waveform delay is added to both modulator
 * and carrier to produce total delays. */
void Sample(x, A, tw, tm, tc, fm, fc, TStep)
	float *x, *A;	/* signal and coefficient array. */
	double tw,	/* waveform time delay (ms) */
	tm,	/* modulator time delay (ms) */
	tc,	/* carrier time delay (ms) */
	fm,fc,	/* modulation and carrier frequency (hz) */
	TStep;
{
	double CD,MD,NREALSTEP,frac1,frac2;
	int n,n2, i[3], p[3];

	CD = (tw + tc) / 1000;
	MD = (tw + tm) / 1000;
	NREALSTEP = NREALPTS * TStep;
	frac1 = (fc*CD + fm*MD)/(fc + fm);	// nothing to do with active fractions
	frac2 = (fc*CD - fm*MD)/(fc - fm);
	
	p[0] = fc*NREALSTEP;
	p[1] = (fc + fm) * NREALSTEP;
	p[2] = (fc - fm) * NREALSTEP;
	
	i[0] = -fc * NREALPTS*CD;
	i[1] = -(fc + fm) * NREALPTS * frac1;
	i[2] = -(fc - fm) * NREALPTS * frac2;
	
	for(n2=0; n2<3; n2++) {
		while (i[n2] >= NREALPTS)
			i[n2] -= NREALPTS;
		while (i[n2] < 0)
			i[n2] += NREALPTS;
	}
	for(n = 0;n < NREALPTS;n++) {
		x[n] =  A[0] * costb[i[0]] + A[1] * costb[i[1]] + A[2] * costb[i[2]];
		for(n2=0; n2<3; n2++)
			if ((i[n2] += p[n2]) >= NREALPTS)
				i[n2] -= NREALPTS;
	}
}

/* checks energy of signal and multiplies by scaling factors to find
 * the proper SPL */
void Normalize(float *x1, float *x2, double TStep)
{
	float in[NREALPTS];
	double sum,Am;
	int n;

	LPFinit(DISP_TSTEP);		/* msec */
	for(n = 0; n < NREALPTS; n++)		/* divide by 2 for fft */
		in[n] = x1[n] / 2;
	fft(in, NCOMPPTS, LOGNCOMPPTS, 1);		/* 1 = FFT */
	multfft(NCOMPPTS, in, LPF, in);
	fft(in, NCOMPPTS, LOGNCOMPPTS, 0);		/* 0 = inverse fft */

	for(sum = 0.0, n = 0; n < NREALPTS; n++)
		sum += in[n];

	/* divide by NREALPTS for invfft */
	sum *= MAXFACT / AMFUDGE;
	Am = 200. * TStep * NREALPTS / sum;
	
	for(n = 0; n < NREALPTS; n++) {
		x1[n] *= Am;
		x2[n] *= Am;
	}
}



ccf *amtone(double SPL, double IID,
			double flc, double frc, double flm, double frm,
			double md, double cd, double wd, double modp)
{
	ccf *Lm;
	float xl[NREALPTS], xr[NREALPTS], al[3], ar[3];
	double DispFL = -1.0, DispFH = -1.0, FRate, fcm, TStep;
	Fraction fraction;
	int fringe, fl, fu;
	extern double strb4_fstep;
	extern int strb4_order;
	extern int strb4_flag;

	FRate = pow(10.0, DISP_FSTEP);
	TStep = DISP_TSTEP * 1e-3;	/* change to seconds */
	flc = fixfreq(flc, DISP_TSTEP);
	flm = fixfreq(flm, DISP_TSTEP);
	frc = fixfreq(frc, DISP_TSTEP);
	frm = fixfreq(frm, DISP_TSTEP);
	printf("%f %f %f\n",flc,flm,wd);
	
	/* find display region */
	fraction = AMtoneActive(SPL, SPL, IID, frc);
	if (isinregion(fcm, frc, fraction) < 0)
		DispFL = DispFH = frc; /* /Frate to make it compatible*/
	else {
		fcm = 200.;
		while(DispFL < 0. && fcm <=20000.) {
			findCosCoef(fcm, flm, flc, al, SPL, modp);
			findCosCoef(fcm, frm, frc, ar, SPL, modp);
			fraction = AMtoneActive(findMS(al), findMS(ar), IID, fcm);
			if (isinregion(fcm, frc, fraction))
					DispFL = fcm;
			fcm *= FRate;
		}
		fcm = 10000.;
		while(DispFH < 0. && fcm > 20.) {
			findCosCoef(fcm, flm, flc, al, SPL, modp);
			findCosCoef(fcm, frm, frc, ar, SPL, modp);
			fraction = AMtoneActive(findMS(al), findMS(ar), IID, fcm);
			if (isinregion(fcm, frc, fraction))
				DispFH = fcm;
			fcm /= FRate;
		}
	}
	
	if (DispFL < 0 || DispFH < 0)
		return(Lm=0);
	
/**************************************************************************
	The next allocation is the basis for Tao's straightness weighting.
	It uses a window centered at a certain f and extending out on both
	sides of f.  The width of the window depends on two variables:
	
	(1) the skip factor, which is straightness_fstep/fstep, or how many freqs
	to skip over when multiplying adjacent amplitudes when computing
	straightness, and
	(2) the order, which is how many amplitudes to look at when computing
	straightness.  Assume that order (==df) is odd so that the window is
	symmetric, e.g., if order=7, the window will extend 3*skip frequencies
	on either side of the "center frequency."
 **************************************************************************/
	 
	fringe = (int) ((strb4_fstep/DISP_FSTEP) + 0.5);
	if (fringe == 0)
		fringe = 1;		/* Round (int)ed result up to 1 */
	fringe *= strb4_flag*(strb4_order-1)/2;	/* Allocate extra freqs for straightness iff flag on */
	
	fl = log10(DispFL)/DISP_FSTEP - fringe;
	fu = log10(DispFH)/DISP_FSTEP + fringe;
	printf("amtone: strb4_order=%d, fringe=%d\n", strb4_order, fringe);
		
	Lm = newccf(fl, fu, DISP_FSTEP,
				(double)DISP_TLOW, (double)DISP_TUP, DISP_TSTEP);

	/********************** Create fractions table *********************/
	activeTable = (Fraction *) calloc(fu - fl + 1, sizeof(Fraction));
	printf("Creating activeTable from %d to %d, 0 to %d\n", fl, fu, fu-fl+1);
		
	beg_freq_loop(Lm)
		findCosCoef(freq, flm, flc, al, SPL, modp);
		findCosCoef(freq, frm, frc, ar, SPL, modp);
		
		/* Get fractions of doubly-/singly-active & spontaneous fiber pairs */
		activeTable[f - Lm->f_low]
			= AMtoneActive(findMS(al), findMS(ar), IID, freq);

		Sample(xl, al, wd, md, cd, flm, flc, TStep);	/* left */
		Sample(xr, ar, 0., 0., 0., frm, frc, TStep);	/* right */
		
		AMRectify(xl, NREALPTS);
		AMRectify(xr, NREALPTS);
		
		Normalize(xr, xl, TStep);		/* find right mag. of functions */
		
		DoCrossCor(Lm, f, xr, xl);

		/**************************
		Frac = active(findMS(ar), freq);
		Frac1 = 1 - Frac;
		if (Frac != 1.0) {
			for(n = Lm->t_low; n < Lm->t_up; n++) {
				Lm->R[f][n] = Frac*(Lm->R[f][n]) + Frac1*CONSTCROSS;
			}
		}
		*************************/
		
	end_freq_loop
	
	return Lm;
}
