/*
*	bilsen.c
*  HISTORY
*	(9-21-88)	TOR - copied from noise.c.
*	(6-15-89)	TOR - adapted for NeXT.
*/

#import <math.h>
#import <stdlib.h>
#import "BiDis.h"
#import "Nerve.h"
#import "ccf.h"
#import <appkit/nextstd.h>

extern Fraction *activeTable;

extern Fraction	NoiseActive(double SPL, double IID,
							double fl, double fh, double fm);
extern double BilsenCCF(double spl, double fl, double fh, double tau, 
				double g, double fm, double *outp, int tup, double tstp);
extern 		nrectify(double vt, double tf, double td, double vn, 
					double *np, int tup, double tstep, double *op), 
			lowpass(double *sig, int tup, double tstep),
			findgausstd(double freq, double *td);
extern int isinregion(double fc, double f0, Fraction thisfraction);
extern execerror(char *s, char *t);

ccf *bilsen(double SPL, double IID, double f0, double bw, double tau, double g)
{
	ccf *Lm;
	Fraction fraction;
	double StimFL = f0 - bw/2., StimFH = f0 + bw/2.;
	double FRate = pow(10.0, DISP_FSTEP);
	double fm, DispFL = -1.0, DispFH = -1.0, Varm, fl, fu;
	double TmpArr[-DISP_TLOW+DISP_TUP+1];
	double *TmpPtr = &TmpArr[-DISP_TLOW];
	int n;
	int fringe;
	extern double strb4_fstep;
	extern int strb4_order;
	extern int strb4_flag;
	/* Set up CCF data structure */

	fraction = NoiseActive(SPL, IID, StimFL, StimFH, fm);
	if (isinregion(fm, f0, fraction) < 0.0)
		DispFL = DispFH = f0;
	else for(fm = 20.; fm <= 20000.; fm *= FRate) {
		fraction = NoiseActive(SPL, IID, StimFL, StimFH, fm);
		if (DispFL < 0.0) {
			if (isinregion(fm, f0, fraction))
				DispFL = fm;
		} else if (!isinregion(fm, f0, fraction)) {
				DispFH = fm/FRate;
				break;
		}
	}
	
	if (DispFL<0 || DispFH<0)
		return(Lm=0);

/**************************************************************************
	The next allocation is the basis for Tao's straightness weighting.
	It uses a window centered at a certain f and extending out on both
	sides of f.  The width of the window depends on two variables:
	
	(1) the skip factor, which is straightness_fstep/fstep, or how many freqs
	to skip over when multiplying adjacent amplitudes when computing
	straightness, and
	(2) the order, which is how many amplitudes to look at when computing
	straightness.  Assume that order (==df) is odd so that the window is
	symmetric, e.g., if order=7, the window will extend 3*skip frequencies
	on either side of the "center frequency."
 **************************************************************************/
	 
	fringe = (int) ((strb4_fstep/DISP_FSTEP) + 0.5);
	if (fringe == 0)
		fringe = 1;		/* Round (int)ed result up to 1 */
	/* Allocate extra freqs for straightness iff flag on */
	fringe *= strb4_flag*(strb4_order-1)/2;
	printf("bilsen noise: strb4_order=%d, fringe=%d\n", strb4_order, fringe);
	
	fl = log10(DispFL)/DISP_FSTEP - fringe;
	fu = log10(DispFH)/DISP_FSTEP + fringe;
	Lm = newccf(fl, fu, DISP_FSTEP,
				(double)DISP_TLOW, (double)DISP_TUP, DISP_TSTEP);
				
	/********************** Create fractions table *********************/
	activeTable = (Fraction *) calloc(fu - fl + 1, sizeof(Fraction));
	printf("Creating activeTable from %.1f to %.1f, 0 to %.1f\n", fl, fu, fu-fl+1);

/****************************************************************************
 	Fill in the crosscorrelation function, but scale by DISP_NFIBERS_STEP
	in straightness routine, not here
 ****************************************************************************/	
	/* Fill it with the crosscorrelation function */	
	beg_freq_loop(Lm)
		/* Put fractions into table */
		activeTable[f - Lm->f_low]
			= NoiseActive(SPL, IID, StimFL, StimFH, freq);

		/* findgausstd(freq, &ts);		Acct. for azimuth, if necessary */

		/* Compute Un-rectified CCF funtion */
		Varm = BilsenCCF(SPL, StimFL, StimFH, tau, g, freq,
			 TmpPtr, DISP_TUP, DISP_TSTEP);
			 
		/* Rectify it */
		nrectify(0.0, 1.0, 0.0,
			 Varm, TmpPtr, DISP_TUP, DISP_TSTEP, Lm->R[f]);
			 
		/* Lowpass filter it */
		lowpass(Lm->R[f], DISP_TUP, DISP_TSTEP);
	end_freq_loop

	return Lm;
}

