/*
*	ccf.h -- This file contains data type declarations for use
*			 by modules using crosscorrelation functions.
*
*	Ver 1.0	(2-19-85)
*	Ver 1.1	(5-23-91)
*/

#import <math.h>

/* Declare the enum type holding the regions here */
typedef enum {
		singlefiber		= 1,
		doublyactive	= 2,
		fixedband 		= 3,
		criticalband	= 4
} Regiontypes;

/* Group the active & inactive fractions here */
typedef struct {
	double n2;		/* fraction of doubly-active fibers */
	double n1;		/* fraction of singly-active fibers */
	double n0;		/* fraction of inactive fibers */
} Fraction;

/* Group the data of cross-correlation function */
typedef struct ccf {
	int f_up, f_low;	/* upper and lower frequency index bounds. */
	int t_up, t_low;	/* upper and lower time index bounds */
	double fstep,tstep;	/* frequency [log10(Hz)] and time (msec) steps. */
	double tdivlabel;	/* spacing for labeling in x axis */
	double fdivlabel;	/* spacing for labeling in y axis */
	double zdivlabel;	/* spacing for labeling in z axis */
	char tlabel[100];	/* label for x axis	*/
	char flabel[50];	/* label for y axis	*/
	char zlabel[50];	/* label for z axis	*/
	char title[128];	/* title of plot	*/
	char info[1024];	/* dump of modules used */
	char *storage;		/* pointer to memory allocated by malloc() */
	double **R;			/* actual CCF samples arranged as R[f][t] */
} ccf;

/* Some revised math routines that check for overflows */
extern double 	Pow(double x, double y),
				Exp(double x),
				Log(double x),
				Log10(double x),
				ctrd_peak(ccf *cr, int f),
				ctrd_dt(ccf *cr, int f),
				ctrd_df(ccf *cr, int t),
				freq_salience(double x, double f),
				pColburn(double x, double f, double t),
				fiber_bw(double f);
		
extern ccf 	*ccfalloc(int a, int b, double c, int d, int e, double f),
			*ccfcopy(ccf *a),
			*ccfread(char *str),
			*ccfsave(ccf *a, char *str),
			*integ_df(ccf *a),
			*integ_dt(ccf *a),
			*minteg_df(ccf *a),
			*diff_df(ccf *a),
			*diff_dt(ccf *a),
			*map(double f(), ccf *a),
			*ccfadd(ccf *a, ccf *b,ccf *c),
			*ccfsub(ccf *a, ccf *b,ccf *c),
			*ccfmul(ccf *a, ccf *b,ccf *c),
			*ccfdiv(ccf *a, ccf *b,ccf *c),
			*ccfpow(ccf *a, ccf *b,ccf *c),
			*ccffill(ccf *a, double d),
			*scalaradd(ccf *a, double d),
			*scalarsub(ccf *a, double d),
			*scalarmul(ccf *a, double d),
			*scalardiv(ccf *a, double d),
			*scalarpow(ccf *a, double d),
			*ccf_nmstr(ccf *a, int df),
			*old_ccf_str(ccf *cr, int df),
			*ccf_str(ccf *cr, int df);
		
extern void	formatccf(ccf *a, double td, double fd, double zd,
					  char *tl, char *fl, char *zl, char *title);
extern void	putinfoccf(ccf *a, char *str), freeccf(ccf *a);

#define newccf(a,b,c,d,e,f)	ccfalloc(dtoi(a), dtoi(b), c, dtoi(d), dtoi(e), f)

#define beg_freq_loop(c) { double frate = -1., freq = -1.; \
						register int f = (c)->f_low; \
						frate = pow((double)10.0, (c)->fstep); \
						for (freq = pow(frate,(double)f); f <= (c)->f_up; \
						f++, freq *= frate) {
#define end_freq_loop   }}

#define beg_f_loop(c)	{ register int f = (c)->f_low; \
			  			for(;f <= (c)->f_up; f++) {
#define end_f_loop   }}

#define beg_t_loop(c)	{ register int t = (c)->t_low; \
						for (;t <= (c)->t_up; t++) {
#define end_t_loop	}}

#define beg_time_loop(c)	{ register int t = (c)->t_low; \
							double time = t * (c)->tstep; \
							for (;t <= (c)->t_up; t++, time += (c)->tstep) {
#define end_time_loop	}}

#define max(a,b)	((a) > (b) ? (a):(b))
#define min(a,b)	((a) < (b) ? (a):(b))
#define dtoi(x)		((int) ((x) < 0.0 ? (x) - 0.5:(x) + 0.5))

/* #define abs(a)	((a) < 0 ? -(a) : (a)) */

#define MAXPATHLEN 256
