/*
*	ccffiler.c
*
*  HISTORY
*	V2.1 (2/3/87)	Tested.
*	V2.1 (2/6/87)	Fixed bug (ptr->R[f] should be ptr->R[f]+t_low)
*	(5-24-89)	TOR - adapted for NeXT.
*/

#include <stdio.h>
#include <ctype.h>
#include "ccf.h"

extern execerror(char *s,char *t);

static double *dp, *rp;
static double **dpp, **rpp;
static ccf *cd, *cr;
static int fmin, fmax, tmin, tmax, lineno;

static char *header = "WIM Crosscorrelation File. Format Version 3.0";

extern FILE	*fopen();

static skiptil (int (*test)(), FILE *fp)
//	int (*test)();
{
	int c, lc = '\0';
	int comment = 0;

	while ( (c = getc(fp)) != EOF && (comment || !(*test)(c)) )
	 {
		if (c == '*' && lc == '/') comment = 1;
		else if (c == '/' && lc == '*') comment = 0;
		else if (c == '\n') lineno++;
		lc = c;
	}

	if (c == '\n') lineno++;

	return c;
}

ccf *ccfsave(ccf *cd, char *fname)
{
	FILE	*fp;
	int	f, items;

	if ((fp = fopen(fname, "w")) == NULL)
		execerror("could not write", fname);

	fprintf (fp, "%s%c", header, 'b');

	fwrite((char *)cd, sizeof(ccf), 1, fp);

	items = cd->t_up - cd->t_low +1;
	for (f = cd->f_low; f <= cd->f_up; f++)
		fwrite(cd->R[f]+cd->t_low, sizeof(double), items, fp);

	if (fp != stdout) fclose(fp);
	return cd;
}


extern int isnum();
static int lineno;

ccf *ccfread (char *fname)
{
	FILE	*fp;
	int	f, t, mode, cc, f_up, f_low, t_low, t_up, items;
	double	fstep, tstep;
	ccf	*tmp, tcc;
	char	*cp = header;

	if ((fp = fopen(fname, "r")) == NULL)
		 execerror("could not read", fname);

	lineno = 1;
	/* Check file type and format version */
	while (*cp != '\0')  {
		if (*cp++ != (cc = getc(fp))) goto badfile;
		if (cc == '\n') lineno++;
	}

	mode = getc(fp);	/* Get mode letter 'a' or 'b' */
	if (mode != 'a' && mode != 'b') goto badfile;

	if (mode == 'b')  {	/* Binary file */

		fread ((char *)&tcc, sizeof(ccf), 1, fp);

		if (feof(fp))  {
			if (fp != stdin) fclose(fp);
			execerror("EOF encountered reading", fname);
		}

		tmp = ccfalloc(tcc.f_low, tcc.f_up, tcc.fstep,
				tcc.t_low, tcc.t_up, tcc.tstep);
		formatccf(tmp,tcc.tdivlabel,tcc.fdivlabel,tcc.zdivlabel,
			  tcc.tlabel,tcc.flabel,tcc.zlabel,tcc.title);
		putinfoccf(tmp,tcc.info);
		items = tcc.t_up - tcc.t_low +1;
		for (f = tmp->f_low; f <= tmp->f_up; f++)
			if (fread(tmp->R[f]+tmp->t_low, sizeof(double),
				  items, fp) != items) {
				freeccf(tmp);
				if (fp != stdin) fclose(fp);
				execerror("EOF encountered reading",
					fname);
			}

		if (fp != stdin) fclose(fp);
		return tmp;
	}

	/*
	* Read an Ascii type file.
	*/

	ungetc (skiptil(isnum, fp), fp);
	if (fscanf (fp, "%d, %d", &f_low, &f_up) < 2)
		goto badfile;

	ungetc (skiptil(isnum, fp), fp);
	if (fscanf (fp, "%lf", &fstep) < 1)
		goto badfile;

	ungetc (skiptil(isnum, fp), fp);
	if (fscanf (fp, "%d, %d", &t_low, &t_up) < 2)
		goto badfile;

	ungetc (skiptil(isnum, fp), fp);
	if (fscanf(fp, "%lf", &tstep) < 1)
		goto badfile;

	tmp = ccfalloc(f_low, f_up, fstep, t_low, t_up, tstep);

	for (f = tmp->f_low; f <= tmp->f_up; f++)  {
		for (t = tmp->t_low; t <= tmp->t_up; t++)  {
			ungetc(skiptil(isnum, fp), fp);
			if (fscanf(fp, "%lf", &(tmp->R[f][t])) == EOF)  {
				freeccf(tmp);
				if (fp != stdin) fclose (fp);
				execerror("EOF encountered while reading",
					fname);
			}
		}
	}

	if (fp != stdin) fclose (fp);
	return tmp;

badfile:
	if (fp != stdin) fclose (fp);
	execerror("error reading file", fname);
}

/*
*	Local support routines.
*/

static isnum(c)
{
	if (c == '.' || c == '-' || isdigit(c)) return 1;
	return 0;
}
