/*
*	ccfcode.c
*
*  HISTORY
*	??		GDS - Created.
*	(7-10-87)	GDS - Modified diff_df() and diff_dt() to catch case
*			when input is only one row or column, respectively.
*	(5-24-89)	TOR - adapted for NeXT.
*   (6-21-89) 	TOR - added formatccf(), putinfoccf() to go with
*						new ccf format.
*    (3-26-90)     XXU - added ccfstr() function to calculate straightness.   
*/

#include <stdio.h>
#import <string.h>
#include "ccf.h"
#import <objc/objc.h>
#import <stdlib.h>
#import "Nerve.h"


extern execerror(char *s,char *t);

extern double DISP_NFIBERS_STEP, peakpick_thresh;
extern Fraction *activeTable;

static double *dp, *rp;
static double **dpp, **rpp;
static ccf *cr;
static int fmin, fmax, tmin, tmax;

double pow(double, double);

extern float elapsed_real_time(int);	// for timing routines
extern float elapsed_user_time(int);	// for timing routines
extern float elapsed_sys_time(int);	// for timing routines
extern	float my_r_time;		// for timing routines
extern	float my_u_time;
extern	float my_s_time;

#define beg_a_loop	{ double **app, *ap; \
			  for(app = a->R+a->f_low; \
		  	      app <= a->R+a->f_up; app++) \
				for(ap = *app+a->t_low; \
				    ap <= *app+a->t_up; ap++) {
#define end_a_loop	}}

#define beg_abr_loop { double **app, **bpp, **rpp, *ap, *bp, *rp; \
		       for(app = a->R+fmin, bpp= b->R+fmin, rpp =r->R+fmin; \
			   app <= a->R+fmax; app++, bpp++, rpp++) \
				for(ap = *app+tmin,bp= *bpp+tmin,rp= *rpp+tmin; \
				    ap <= *app+tmax; ap++, bp++, rp++) {
#define end_abr_loop }}

ccf *map(double (*func)(), ccf *r)	/* map built-in(x, f, t) over ccf */
{
	beg_freq_loop(r)
		beg_time_loop(r)
			r->R[f][t] = (*func)(r->R[f][t], freq, time);
		end_time_loop
	end_freq_loop
	return r;
}

/****************** Set up fmin, fmax, etc. for overlap ********************/
int findoverlap(ccf *a, ccf *b, ccf *c)
{
	fmin = max(a->f_low, b->f_low);
	fmax = min(a->f_up, b->f_up);
	tmin = max(a->t_low, b->t_low);
	tmax = min(a->t_up, b->t_up);
	if (c) {
		fmin = max(fmin, c->f_low);
		fmax = min(fmax, c->f_up);
		tmin = max(tmin, c->t_low);
		tmax = min(tmax, c->t_up);
	}
	if (fmin > fmax || tmin > tmax)		/* no overlap */
		return 1;
	return 0;
}

ccf *ccfcopy(ccf *a)		/* make a copy of ccf on stack */
{
	ccf *b = ccfalloc(a->f_low, a->f_up, a->fstep, a->t_low, a->t_up, a->tstep);

	beg_f_loop(a)
		beg_t_loop(a)
			b->R[f][t] = a->R[f][t];
		end_t_loop
	end_f_loop

	return b;
}

ccf *ccffill(ccf *cr, double d)	/* assign a constant to all elements of a ccf */
{

	for (rpp =cr->R +cr->f_low; rpp <= cr->R +cr->f_up; rpp++)
		for (rp = *rpp +cr->t_low; rp <= *rpp +cr->t_up;)
			(*(rp++)) = d;
	return cr;
}

ccf *ccfadd(ccf *a, ccf *b, ccf *r)
{
	if (findoverlap(a, b, r))	/* no overlap ==> no-op */
		return r;

	if (r == 0)
		r = ccfalloc(fmin, fmax, a->fstep, tmin, tmax, a->tstep);

	beg_abr_loop
		*rp = *ap + *bp;
	end_abr_loop
	return r;
}

ccf *ccfsub(ccf *a, ccf *b, ccf *r)
{
	if (findoverlap(a, b, r))	/* no overlap ==> no-op */
		return r;

	if (r == 0)
		r = ccfalloc(fmin, fmax, a->fstep, tmin, tmax, a->tstep);

	beg_abr_loop
		*rp = *ap - *bp;
	end_abr_loop
	return r;
}
ccf *ccfmul(ccf *a, ccf *b, ccf *r)
{
	if (findoverlap(a, b, r))	/* no overlap ==> no-op */
		return r;

	if (r == 0)
		r = ccfalloc(fmin, fmax, a->fstep, tmin, tmax, a->tstep);

	beg_abr_loop
		*rp = (*ap) * (*bp);
	end_abr_loop
	return r;
}
ccf *ccfdiv(ccf *a, ccf *b, ccf *r)
{
	if (findoverlap(a, b, r))	/* no overlap ==> no-op */
		return r;

	if (r == 0)
		r = ccfalloc(fmin, fmax, a->fstep, tmin, tmax, a->tstep);

	beg_abr_loop
		*rp = *ap / *bp;
	end_abr_loop
	return r;
}
ccf *ccfpow(ccf *a, ccf *b, ccf *r)
{
	if (findoverlap(a, b, r))	/* no overlap ==> no-op */
		return r;

	if (r == 0)
		r = ccfalloc(fmin, fmax, a->fstep, tmin, tmax, a->tstep);

	beg_abr_loop
		*rp = Pow(*ap,*bp);
	end_abr_loop
	return r;
}

ccf *diff_dt(ccf *cd)
{
	register int t0, t1;
	register double dtstep;

	cr = ccfalloc(cd->f_low, cd->f_up, cd->fstep,
			cd->t_low, cd->t_up, cd->tstep);
	t0 = cd->t_low;
	t1 = cd->t_up;
	if (t1 <= t0) return ccffill(cr, (double)0.0);

	dtstep = 2.0 * (cd->tstep);
	for (dpp = cd->R +cd->f_low, rpp = cr->R +cr->f_low;
	     dpp <= cd->R +cd->f_up; dpp++, rpp++)  {
		dp = *dpp +t0;
		rp = *rpp +t0;
		*rp++ = (dp[1] - *dp)/cd->tstep;
		for (dp++; dp < *dpp +t1; dp++)
			*rp++ = (dp[1] - dp[-1])/dtstep;
		*rp = (*dp - dp[-1])/cd->tstep;
	}
	return cr;
}

ccf *diff_df(ccf *cd)
{
	register int f0, f1, f, t;
	register double dfstep;

	cr = ccfalloc(cd->f_low, cd->f_up, cd->fstep,
			cd->t_low, cd->t_up, cd->tstep);
	f0 = cd->f_low;
	f1 = cd->f_up;

	if (f1 <= f0) return ccffill(cr, (double)0.0);

	dfstep = 2.0 * (cd->fstep);
	for(t = cd->t_low; t <= cd->t_up; t++) {
		cr->R[f0][t] = (cd->R[f0+1][t] - cd->R[f0][t])/cd->fstep;
		for (f = f0+1; f < f1; f++)
			cr->R[f][t] = (cd->R[f+1][t]-cd->R[f-1][t])/dfstep;
		cr->R[f][t] = (cd->R[f][t] -cd->R[f-1][t])/cd->fstep;
	}
	return cr;
}

ccf *integ_df(ccf *cd)
{
	register int f, t;

	cr = ccfalloc(0, 0, (double)0.0, cd->t_low, cd->t_up, cd->tstep);

	rp = *(cr->R) +cr->t_low;
	for (t = cd->t_low; t <= cd->t_up; t++)  {
		*rp = 0.0;
		for (f = cd->f_low; f <= cd->f_up; f++)
			(*rp) += cd->R[f][t];
		(*rp++) *= cd->fstep;
	}
	return cr;
}

ccf *minteg_df(ccf *cd)
{
	register int f, t;

	cr = ccfalloc(0, 0, (double)0.0, cd->t_low, cd->t_up, cd->tstep);

	rp = *(cr->R) + cr->t_low;
	/* printf ("\n f_low = %d, f_up = %d, fstep = %f\n", cd->f_low, cd->f_up, cd->fstep); */
	for (t = cd->t_low; t <= cd->t_up; t++)  {
		*rp = 1.0;
		for (f = cd->f_low; f <= cd->f_up-3; f++)
			(*rp) *= cd->R[f][t];
 		(*rp++) *= cd->fstep;
	}
	return cr;
}

ccf *integ_dt(ccf *cd)
{
	cr = ccfalloc(cd->f_low, cd->f_up, cd->fstep, 0, 0, (double)0.0);

	for (dpp =cd->R +cd->f_low, rp = *(cr->R +cr->f_low);
	     dpp <= cd->R +cd->f_up; dpp++) {
		*rp = 0.0;
		for (dp = *dpp +cd->t_low; dp <= *dpp +cd->t_up;)
			(*rp) += (*(dp++));
		(*rp++) *= cd->tstep;
	}
	return cr;
}

ccf *ccfconvolve(ccf *cr, ccf *cd, ccf *co)
{
	int t, f, t1, f1, f1max, t1max;

	if (cd->fstep != cr->fstep || cd->tstep != cr->tstep)
		execerror("incompatible ccf step sizes", (char *)0);
	if (co == 0)
		co = ccfalloc(cr->f_low +cd->f_low, cr->f_up +cd->f_up,
				cr->fstep,
			      cr->t_low +cd->t_low, cr->t_up +cr->t_up,
				cr->tstep);
	else {
		co->fstep = cr->fstep;
		co->tstep = cr->tstep;
	}

	for (f = co->f_low; f <= co->f_up; f++) {
		for (t = co->t_low; t <= co->t_up; t++)  {

			co->R[f][t] = 0.0;
			f1max = min(cr->f_up, f - cd->f_low);
			for (f1 =max(cr->f_low, f - cd->f_up); f1 <= f1max;
			     f1++) {
				t1max = min(cr->t_up, t -cd->t_low);
				for (t1 =max(cr->t_low, t -cd->t_up);
				     t1 <= t1max; t1++)
					co->R[f][t] +=
					     cr->R[f1][t1] *cd->R[f-f1][t-t1];
			}
			co->R[f][t] *= cr->fstep * cr->tstep;
		}
	}
	return co;
}

ccf *scalaradd(ccf *a, double d)
{
	beg_a_loop
		(*ap) += d;
	end_a_loop
	return a;
}

ccf *scalarsub(ccf *a, double d)
{
	beg_a_loop
		(*ap) -= d;
	end_a_loop
	return a;
}

ccf *scalarmul(ccf *a, double d)
{
	beg_a_loop
		(*ap) *= d;
	end_a_loop
	return a;
}

ccf *scalardiv(ccf *a, double d)
{
	beg_a_loop
		(*ap) /= d;
	end_a_loop
	return a;
}

ccf *scalarpow(ccf *a, double d)
{
	beg_a_loop
		(*ap) = Pow(*ap,d);
	end_a_loop
	return a;
}

char *tmalloc(unsigned s)
{
	//char *p = calloc(s, 1);
	char *p = malloc(s);
	if (p == NULL)
		execerror("out of memory", (char *)0);
	return p;
}

ccf *ccfdummy(void)
{
	ccf *cp;

	cp = (ccf *) tmalloc(sizeof(ccf));
	cp->storage = NULL;
	cp->f_low = 0;
	cp->f_up = 0;
	cp->fstep = 0;
	cp->t_low = 0;
	cp->t_up = 0;
	cp->tstep = 0;
	cp->R = 0;
	return cp;
}

ccf *ccfalloc(int fl, int fu, double dfs, int tl, int tu, double dts)
{
	int n;
	unsigned nrows, ncols;
	double *ptr;
	ccf *cp;

	nrows = fu - fl + 1;
	ncols = tu - tl + 1;
	if ((int)nrows <= 0 || (int)ncols <= 0)
		return 0;	// Return nil pointer if bad limits
			
	/* printf("noise:  fl=%d, fu=%d ==> nrows=%d; tl=%d, tu=%d ==> ncols=%d\n", fl, fu, nrows, tl, tu, ncols); */
	
	cp = (ccf *) tmalloc(sizeof(ccf));
	cp->storage = tmalloc(sizeof(double *) * nrows
				+ sizeof(double) * ncols * nrows);

	cp->f_low =	fl;
	cp->f_up = fu;
	cp->fstep = dfs;
	cp->t_low = tl;
	cp->t_up = tu;
	cp->tstep = dts;
	cp->R = ((double **)(cp->storage)) - fl;
	for (n = fl, ptr = (double *)(((double **)(cp->storage)) + nrows);
	     n <= fu; n++, ptr += ncols)
		cp->R[n] = ptr - tl;

	return cp;
}

void freeccf(ccf *p)
{
	free(p->storage);
	p->storage = NULL;

}

#define FUNCTION(x) (temp->R[f][(x)])


/*
-5    -4    -3    -2    -1     0    1     2     3     4     5    :t
-200  -160  -120  -80   -40    0    40    80    120   160   200  :time
*/

/************************ CCF-peak finding routine ************************/
double ctrd_peak(ccf *temp, int f)
{
	int t, u;
	double peak_location = 0, peak_amp1 = -999999, peak_amp2 = -999999;

	if (f < temp->f_low || f > temp->f_up)
	  execerror("frequency out of bounds", (char *)0);

	for (t = temp->t_low + 1; t <= temp->t_up - 1; t++)  {
	
		if (FUNCTION(t) > FUNCTION(t-1)  &&  FUNCTION(t) >= FUNCTION(t+1)) {
			for (u = 2; u <= (temp->t_up - t); u++) {		// Check for plateau
				if (FUNCTION(t) > FUNCTION(t + u))		// Plateau ends
					break;
			}
			if (u > temp->t_up - t)
				break;	//since plateau hits end, don't check for more peaks
				
			if (FUNCTION(t + (u-1)/2) >= peak_amp1) {	// Biggest max?
				t += (u-1)/2;	// advance counter to peak
				peak_amp2 = peak_amp1;		// Swap peak values
				peak_amp1 = FUNCTION(t);
				peak_location = t * temp->tstep;	// Time loc
				printf("# overrode max_peak   at %d with %.4g\n", t, peak_amp1);
			} else
			if (FUNCTION(t + (u-1)/2) >= peak_amp2) {	//Next biggest max?
				t += (u-1)/2;	// Advance counter to peak
				peak_amp2 = FUNCTION(t);
				printf("# overrode 2-ary_peak at %d with %.4g\n", t, peak_amp2);
			} else {
				printf("# overrode nothing    at %d with %.4g\n", t, FUNCTION(t+(u-1)/2));
			}
		}	//End of local peak test (if)
		
	}	//end of t loop
	
	
	if (peak_amp2 >= peakpick_thresh * peak_amp1) {	//peaks too similar
		peak_location = -999999;
		printf("peak-picker threshold is %f\n", peakpick_thresh);
	}
	
	return peak_location;
}


double ctrd_dt(ccf *cr, int f)
{
	int t;
	double time, avg = 0.0, norm = 0.0;

	if (f < cr->f_low || f > cr->f_up)
	  execerror("frequency out of bounds", (char *)0);

	for (t = cr->t_low, time = cr->t_low * cr->tstep; t <= cr->t_up;
	     t++, time += cr->tstep)  {

		norm += cr->R[f][t];
		avg += cr->R[f][t] * time;
	}
	return avg/norm;
}

double ctrd_df(ccf *cr, int t)
{
	int f;
	double freq, avg = 0.0, norm = 0.0;

	if (t < cr->t_low || t > cr->t_up)
		execerror("time index out of bounds", (char *)0);
	
	for (f = cr->f_low, freq = cr->f_low * cr->fstep; f <= cr->f_up;
	     f++, freq += cr->fstep)  {

		norm += cr->R[f][t];
		avg += cr->R[f][t] * freq;
	}

	return Pow(10.0, avg/norm);	/* Convert back to Hz. */
}

/* TOR - function inserts string into end of ccf info label */
void putinfoccf(ccf *cr, char *str)
{
	int len;
	len = strlen(cr->info);
	strcpy(&(cr->info[len]),str);
}

/* TOR - this function sets the divlabel and label fields of the ccf */
void formatccf(ccf *cr, double td, double fd, double zd,
			   char *tl, char *fl, char *zl, char *title)
{
	cr->tdivlabel = td;
	cr->fdivlabel = fd;
	cr->zdivlabel = zd;
	strcpy(cr->tlabel,tl);
	strcpy(cr->flabel,fl);
	strcpy(cr->zlabel,zl);
	strcpy(cr->title,title);
}

/**************************************************************************
	XXU  - calculate the straightness
	df  - the bandwidth of frequency counter
	strfstep - the frequency step of the straightness algorithm

	This routine has been slightly modified, because the stimulus routines
	now return the CCF given that both fibers are active.  The active
	fraction table activeTable has the fractions of active fibers.  If one
	or both of the fibers is/are inactive, the CCF is a constant, and we
	can use activeTable to allocate these CCFs in the correct proportions.
	
 **************************************************************************/
ccf *old_ccf_str(ccf *cr, int df)	/* df is the order */
{
	extern double DISP_FSTEP;	/* The value of fstep */
	extern double strb4_fstep;
	extern int strb4_flag;
	double sum = 1.0;
	int f, t, con, skip, fringe;
	ccf *cstr;

	puts("Old straightness routine");
/**************************************************************************
	Shape CCF by including contributions of singly-active and inactive
	fiber pairs.
 **************************************************************************/		
	beg_t_loop(cr)
		beg_f_loop(cr)
		// This loop could be faster, but doesn't need to be.
			cr->R[f][t] *= activeTable[f - cr->f_low].n2;
			cr->R[f][t] +=
				SINGLY_ACTIVE_PROD * activeTable[f - cr->f_low].n1 +
				SPONT_ACTIVE_PROD  * activeTable[f - cr->f_low].n0;
			cr->R[f][t] *= DISP_NFIBERS_STEP;
		end_f_loop
	end_t_loop


/***********************************************************************
	Here, skip is how many freq elements to skip over, i.e. the number of
	elements at the beginning and end of the matrix is
	fringe=skip*(half of the order-1)/2
 ***********************************************************************/
	skip = (int) ((strb4_fstep/DISP_FSTEP) + 0.5);
	if (skip == 0)
		skip = 1;
	skip *= strb4_flag;
	fringe = skip * (df - 1)/2;
	
/****************************************************************************
	Create a *ccf to hold the result of straightness.  The straightness
	routine looks at the fringes, but isn't ever centered in the fringe
	area, so the *cstr will be exactly like *cr, except that it'll be
	smaller by 2*fringe.
 ****************************************************************************/
	cstr = newccf(cr->f_low + fringe, cr->f_up - fringe, cr->fstep,
				  cr->t_low, cr->t_up, cr->tstep);
				  
	if (cstr == (ccf *)0)	// Bad CCF allocation, so return and quit
		return cstr;
		
	for(t = cr->t_low; t <= cr->t_up; t++) {
		for (f = cr->f_low + fringe; f <= cr->f_up - fringe; f++) {
			/* Calculate straightness using fstep*skip increments */
			/* Now we do a 'convolution' */
			for(con = -fringe; con <= fringe; con += skip) {
				sum *= cr->R[f + con][t];
			}
			cstr->R[f][t] = sum;	/* Stick each product into str matrix */
			sum = 1.0;
		}
	}
	
	freeccf(cr);
	return cstr;	/* Send back straightness matrix */
}

/******************* This routine is no longer in use *********************/
ccf *ccf_nmstr(cr, df)  /*normalized straightness */
	ccf *cr;
	int df;
{
          double sum,dev;
	  double max;
	  int con;
         sum = 1.0;

       beg_time_loop(cr)
           max=0.0;
               beg_freq_loop(cr)
	           if(cr->R[f][t]>=max) max=cr->R[f][t];
	       end_freq_loop
	       if(max == 0.0)max = 1.0;
	       dev=max;
	/*       printf("\n max = %f \n",max); */
          {double frate = -1.,freq = -1.;\
               int f = (cr)->f_low;\
               frate = pow((double)10.0,(cr)->fstep);\
               for (freq=pow(frate,(double)f);f<=(cr)->f_up-df;\
               f++,freq*=frate){	       
              for(con=0;con<=df;con++)
               sum*=(cr->R[f+con][t]/dev);
             cr->R[f][t]=sum;
               sum=1.0;
               end_freq_loop
          end_time_loop

	return cr;	/* */
}



