
/*
*	DetectTinN.c
*  HISTORY
*	(2-13-87)	GDS - created.
*	(2-24-87)	GDS - Eliminated line which chose between NOISE_SLOPE()
*			and TONE_SLOPE() in call to rectify().
*	(3-4-87)	GDS - Changed CalcDiff() to do a search for N0Spi case
*			too.  Also changed default SrchHi from 90 to 65.
*	(3-13-87)	GDS - Implemented v-th law halfwave rect.
*	(3-17-87)	GDS - Added OneFiber Var. to give you choice.
*	(4-6-87)	GDS - Altered to us REGION module. And to changed
*			to compute average Qhat, not sum of Qhat. Eliminated
*			StaticPerc variable.
*	(4-28-87)	GDS - Clip negative values in Np[t] and TNp[t] to 0.
*			(And rearranged Debugging code.  This should not affect
*			predictions.)
*	(4-29-87)	GDS - Added SameNoise Flag and code so that IntN is not
*			necessarily recalculated during each call to
*			CalcDprime().  (Date in module comment was not changed
*			because these mod.'s should not alter predictions.)
*	(5-9-87)	GDS - Corrected SameNoise concept since it depends on
*			REGION->RegType.  (Predictions from 4-29 to 5-9 are
*			valid if only one fiber was used (i.e.,RegType == 0))
*			Also added check for IntTN or IntN <= 0.0
*	(6-19-87)	GDS - Modified so Debug causes printing of
*			tau vs. N TN rather than tau vs. (N-TN). Does not
*			affect predicitons.
*	(6-21-87)	GDS - Modified so DebugOn also prints IntN and IntTN.
*			Does not affect predictions.
*	(6-20-89)	TOR - adapted for NeXT.
*        (4-25-90) XXU - added function CDprime .
*/
#import <stdio.h> 
#import <math.h>
#import "ccf.h"
#import "BiDis.h"
#import "Nerve.h"
#import "s0spi.data" 

#define RADPERDEG	(PI/180.0)

#define WARNING		1	/* Define to enable TSPLtooBig warning msg. */


extern double 	ToneNoiseActive(double IID, double tspl, double tf,
								double nspl, double nfl, double nfh,
								double fm),
				NoiseActive(double SPL, double IID, double fl, double fh,
							double fm),
				NoiseCCF(double SPL, double fl, double fh,
                   		double td, double ps, double fm,
						double *outp, int tup, double tstep);
extern double 	HmSqd(double f, double fm),
				ptau(double t),
				gauss(double t),
				GSqd(double f),
				Pow(double x,double y),
				Exp(double x),
				Log10(double x);
extern int 	Xisinregion(double fc, double f0, double frac),
			ptauinit(double f);
extern 	nrectify(double vart,double tf,double ttd,double varn,
				 double *np, int tup, double tstep, double *op),
		lowpass(double *sig, int tup, double tstep),
		execerror(char *s, char *t);
		
static int	TSPLtooBig = 0;		/* warning flag set by CalcDprime() */
					/* when IntTN > IntN */

static int	SameNoise = 0;		/* Flag set in Detect() to indicate */
					/* when IntN will have the same value*/
					/* as the previous call to CalcDprime*/
double TSPL;
double CalcDprime(double IID, double Tps, double Tf, double TSPL,
				  double NSPL, double Nfl, double Nfh, double Ntd, double Nps,
				  double ThDp, double *UnRctp, double *TNp, double *Np)
{
	double	Frac2, Frac1, Frac0, FracTN, FracN,
			IntTN = 0.0,DpSqd,VarTm, VarT, Varm, fm,tem;
	double	FRate = Pow(10.0, DISP_FSTEP),
			ps = Tps * RADPERDEG,
			Fstrt = 20., Fstop = 10000.;

	int	t, freqdep = ptauinit(500.);
	short done = 0, DoNoiseToo = 1;
	static double VarNm, IntN;

	errno = 0;
 
	if (!SameNoise)
		IntN = 0.0;
	if (Xisinregion(fm, Tf, FracTN) < 0) {
		Fstop = Fstrt = Tf;
		if (SameNoise)
			DoNoiseToo = 0;
	}
	VarT = Pow(10.0, TSPL/10.);
	for (fm = Fstrt; fm <= Fstop; fm *= FRate) {
		FracTN = ToneNoiseActive(IID, TSPL, Tf, NSPL, Nfl, Nfh, fm);
		if (!Xisinregion(fm, Tf, FracTN*FracTN)) {
			if (done)
				break;
			continue;
		}
		done = 1;
		if (freqdep)
			ptauinit(fm);
		VarTm = HmSqd(Tf, fm) * VarT;
		if (DoNoiseToo)	/* Compute unrectified noise ccf */
			VarNm = NoiseCCF(NSPL, Nfl, Nfh, Ntd, Nps, fm,
					 UnRctp, DISP_TUP, DISP_TSTEP);
					 
		/* Compute rectified tone+noise ccf (AGC included) */
		nrectify(VarTm, Tf, (Tps/360.)*(1000./Tf),
			VarNm, UnRctp, DISP_TUP, DISP_TSTEP, TNp);
			
		/* lowpass filter it */
		lowpass(TNp, DISP_TUP, DISP_TSTEP);
		
		Frac2 = FracTN*FracTN;
		Frac0 = (1.-FracTN)*(1.-FracTN);
		Frac1 = 1. - Frac2 - Frac0;
		/* Include singly-active and inactive fibers in correct
		*  proportions. */
		Frac2 *= DISP_NFIBERS_STEP;
		Frac0 = (Frac1 *NERVE_SPONT_RATE*NERVE_ACTIVE_RATE +
			 	Frac0 *NERVE_SPONT_RATE*NERVE_SPONT_RATE) *
				DISP_NFIBERS_STEP;
		for (t = -DISP_TUP; t <= DISP_TUP; t++) {
			if (TNp[t] < 0.0)
				TNp[t] = 0.0;
           /* XXU - added gaussian intensity weighting */
		   tem =  (ptau((double)t*DISP_TSTEP) *(Frac2 *TNp[t] + Frac0));
			IntTN +=tem;    /*  *gauss((double)t*DISP_TSTEP);*/
		}
		if (!SameNoise) {/* Compute integral of noise alone case */
			FracN = NoiseActive(NSPL, IID, Nfl, Nfh, fm);

			/* Rectify it */
			nrectify(0.0, 500.0, 0.0, VarNm, UnRctp, DISP_TUP, DISP_TSTEP, Np);

			/* Lowpass filter it */
			lowpass(Np, DISP_TUP, DISP_TSTEP);
			Frac2 = FracN*FracN;
			Frac0 = (1.-FracN)*(1.-FracN);
			Frac1 = 1. - Frac2 - Frac0;
			
			/* Include singly-active and inactive fibers in correct
			*  proportions. */
			Frac2 *= DISP_NFIBERS_STEP;
			Frac0 = (Frac1 *NERVE_SPONT_RATE*NERVE_ACTIVE_RATE +
				 Frac0 *NERVE_SPONT_RATE*NERVE_SPONT_RATE) *
				DISP_NFIBERS_STEP;
			for (t = -DISP_TUP; t <= DISP_TUP; t++) {
				if (Np[t] < 0.0)
					Np[t] = 0.0;
                    /* XXU - added gaussian intensity weighting */	
				tem= (ptau((double)t*DISP_TSTEP)*(Frac2 *Np[t] + Frac0));			
				IntN += tem;     /* *gauss((double)t*DISP_TSTEP);  */
			}
		}
		/*
		*  And now for a bunch of debugging garbage ...
		*  (search ahead to "debugging" to skip over it)
		*/
	}
	IntTN *= DISP_TSTEP/1000.;
	if (!SameNoise)
		IntN *= DISP_TSTEP/1000.;
	/* Set warning flag if necessary */
	TSPLtooBig = (IntTN > IntN) ? 1 : 0;
	/* compute Dprime term and add it to sum */
	if (IntN <= 0.0)
		DpSqd = 2.*IntTN;	/* 2 comes from avg. var. */
	else if (IntTN <= 0.0)
		DpSqd = 2.*IntN;	/* 2 comes from avg. var. */
	else DpSqd = (IntN - IntTN)*(IntN-IntTN)/sqrt(IntN*IntTN);
	return sqrt(DpSqd)/ThDp;
}

double Detect(double IID,
			  double NSPL, double Nfl, double Nfh, double Ntd, double Nps,
			  double TSPL,double Tf, double Tps, double ThDp, double ThDpTol,
			  double SrchLo, double SrchHi)
{
	double Lo = SrchLo+TSPL, Hi = SrchHi+TSPL;
	double LoDp, HiDp, Dp;
	double	NArray[-DISP_TLOW+DISP_TUP+1],
			TNArray[-DISP_TLOW+DISP_TUP+1],
			UnRctArray[-DISP_TLOW+DISP_TUP+1];
	double	*Np = &NArray[-DISP_TLOW],
			*TNp = &TNArray[-DISP_TLOW],
			*UnRctp = &UnRctArray[-DISP_TLOW];



/*	TSPL = Lo;
	LoDp = CalcDprime(IID, Tps,Tf,TSPL,NSPL,Nfl,Nfh,Ntd,Nps,
						ThDp,UnRctp,TNp,Np);
	TSPL = Hi;
	HiDp = CalcDprime(IID, Tps,Tf,TSPL,NSPL,Nfl,Nfh,Ntd,Nps,
						ThDp,UnRctp,TNp,Np);
	if (LoDp > 1.0+ThDpTol) {
		printf("SrchLo = %.8g is to hi (Dp = %.g)\n",
			Lo, LoDp);
		execerror("Bad choice of lower detection bound", (char *)0);
	}
	if (HiDp < 1.0-ThDpTol) {
		printf("SrchHi = %.8g is to low (Dp = %.g)\n",
			Hi, HiDp);
		execerror("Bad choice of upper detection bound", (char *)0);
	}

	TSPL = (1.0 - HiDp)*(Hi-Lo)/(HiDp-LoDp) + Hi;
*/
	while((Hi- Lo) > .1) {
		Dp = CalcDprime(IID, Tps,Tf,TSPL,NSPL,Nfl,Nfh,Ntd,Nps,
						ThDp,UnRctp,TNp,Np);
		SameNoise = 1;
		if (TSPLtooBig) {
#ifdef WARNING
			fprintf(stderr,"warning TSPL=%g is too big\n", TSPL);
#endif
			Hi = TSPL;
		} else if (Dp < 1.0-ThDpTol) { 	/* Below Threshold */
			Lo = TSPL;
/*			LoDp = Dp;
			TSPL = (1.0 - HiDp)*(Hi-Lo)/(HiDp-LoDp) + Hi;
*/
		} else if (Dp > 1.0+ThDpTol) { /* Above Threshold */
			Hi = TSPL;
/*			HiDp = Dp;
			TSPL = (1.0 - HiDp)*(Hi-Lo)/(HiDp-LoDp) + Hi;
*/
		} else {	/* At threshold */
			SameNoise = 0;
			return TSPL;
		}
		TSPL = (Hi+Lo)/2.0;
	}
	SameNoise = 0;
	execerror("Detect didn't converge", (char *)0);
}

double CalcDiff(double IID,
				double NSPL, double Nfl, double Nfh, double Ntd, double Nps,
				double TSPL, double Tf, double Tps,
				double ThDp, double ThDpTol,
				double SrchLo, double SrchHi)
{
	double oldThDp = ThDp, oldTSPL = TSPL;
	double oldTps = Tps, oldNps = Nps;
	double diff;
	double	NArray[-DISP_TLOW+DISP_TUP+1],
	TNArray[-DISP_TLOW+DISP_TUP+1],
	UnRctArray[-DISP_TLOW+DISP_TUP+1];
	double	*Np = &NArray[-DISP_TLOW],
		*TNp = &TNArray[-DISP_TLOW],
		*UnRctp = &UnRctArray[-DISP_TLOW];

	SameNoise = 0;

	Tps = 180.0;
	Nps = 0.0;
	ThDp = 1.0;
	ThDp = CalcDprime(IID, Tps, Tf, TSPL, NSPL, Nfl, Nfh, Ntd, Nps,
					  ThDp, UnRctp, TNp, Np);
	Tps = 0.0;
	Nps = 180.0;
	diff = Detect(IID, NSPL, Nfl, Nfh, Ntd, Nps, TSPL, Tf, Tps,
				ThDp, ThDpTol, SrchLo, SrchHi) - oldTSPL;

	TSPL = oldTSPL;
	Tps = oldTps;
	Nps = oldNps;
	ThDp = oldThDp;
	return diff;
} 

double CDprime(double IID,
			   double NSPL, double Nfl, double Nfh, double Ntd, double Nps,
			   double TSPL, double Tf, double Tps, double ThDp, double ThDpTol,
			   double SrchLo, double SrchHi)
{
	double	NArray[-DISP_TLOW+DISP_TUP+1],
			TNArray[-DISP_TLOW+DISP_TUP+1],
			UnRctArray[-DISP_TLOW+DISP_TUP+1];
	double thdp;
	double	*Np = &NArray[-DISP_TLOW],
			*TNp = &TNArray[-DISP_TLOW],
			*UnRctp = &UnRctArray[-DISP_TLOW];

	SameNoise = 0;

	thdp = CalcDprime(IID, Tps,Tf,TSPL,NSPL,Nfl,Nfh,Ntd,Nps
					,ThDp,UnRctp,TNp,Np);
	return thdp;
} 



double gam(double x)
{
	return (((0.00010469664* x + 2.7105054e-20) *x - 0.016071696) *x
		- 8.6736174e-19) *x + 0.98273013;
}

double CalcColbDiff(double f)
{
	double num, den, k, prvden, g, finv;

	g = 10. *GSqd(f);

	ptauinit(f);
	finv = (1.0/f) * 1000.0;
	prvden = 0.0;
	num = ptau(prvden);
	den = 2.0 * ptau(.5 * finv) * Exp(-g * (1 - gam(.5*finv)));
	for (k = finv; prvden/den <.99999; k=k+finv, prvden =den)
	  {
	  num += 2.0 * ptau(k) * Exp(-g * (1 - gam(k)));
	  den += 2.0 * ptau(.5 * finv+k) * Exp(-g*(1.0-gam(.5*finv +k)));
	  }
	return 10. *Log10(sqrt(num/den));
} 

double CalcColbError(void)
{
	double prevfreq = 0., prevratio, error = 0.;
	int i;

	for (i = 0; i < arysize(s0spidata); i++) {
		if (prevfreq != s0spidata[i].freq) {
			prevfreq = s0spidata[i].freq;
			prevratio = CalcColbDiff(prevfreq);
		}
		error += ((prevratio - s0spidata[i].ratio)
			  *(prevratio - s0spidata[i].ratio));
	}
	return error;
}

