/*
*	FreqSal.c -- Frequency salience module.
*  HISTORY
*	(2-18-87??)	GDS - created.
*	(6-9-87)	GDS - Added option to allow for weighting by
*			fm*FreqSal(fm).
*	(7-30-87)	GDS - Changed Default for "On" to 0.0 (i.e. off).
*       (4-26-90)       XXU - added freq_sal_map
*/
#include <math.h>
#include "ccf.h"

#define ON		1.0
#define WbyF	0.0

extern int errno;

/*
*	Frequency salience weighting function.
*	Cubic approximation of Raatgever data.
*/

#define b3	-3.99154e-8		/* Coefficient of term of order 3 */
#define b2	1.12586e-4		/* Coefficient of term of order 2 */
#define b1	-9.38272e-2		/* Coefficient of term of order 1 */
#define b0	35.017			/* Constant (Not used) */

static double oldf = -1.0;

double freq_salience(double x, double f)	/* f is frequency in Hz */
{
	static double rval;

	if (ON == 0.0) return x;

	if (f != oldf && (oldf < 1200.0 || f < 1200.0)) {
		oldf = f;
		if (f > 1200.0) f = 1200.0;
		rval = pow( 10.0, (((b3*f + b2)*f + b1)*f)/(-10.) );
		errno = 0;
	}
	if (WbyF != 0.0) return f*rval*x;
	return rval*x;
}

double FreqSal(double f)
{
	return freq_salience(1.0, f);
}

static double prevf= -1.0;

double freq_sal_map(double x, double f, double t) 	/* f in Hz, t in msec */
{
	static double y;
	
	if (f != prevf) {
		y = freq_salience(1.0, f);
		prevf = f;
		/* printf("\n y = %f\n",y);  */
	}
	return x*y;
}

