
/*    (7-19-89)       TOR - created support functions for intensity difference
*      (1-?-90) XXU added test mode for mean and variance
*      (3-28-90) XXU added new functions gaussmap2 and  newPtauLi .
*       (3-?-90)  XXU changed program structure  to allow iid as a variable.
*      (4-20-90) XXU added  function gauss(double t) for detection.
*       (4-20-90) XXU fixed a bug by change gauss_array to double.
*/

#import <stdio.h>
#import <string.h>
#import <stdlib.h>
#import "ccf.h"
#import "BiDis.h"
#import "Nerve.h"
#import "Gaussfunc.h"

#define FCONST 10	/* 10 Hz/pt */
#define IIDCONST 5	/* 5 pts/dB */
/* Path to look for tables in after checking ../tables path; also defined
   in interp.m */
#define PVMODEL_IIDTABLE "PVMODEL_IIDTABLE"
#define PVMODEL_AZTABLE "PVMODEL_AZTABLE"

extern id gaussfunction;
extern double mean, deviation;

static double gauss_array[1024];
struct azstruct {
	double iid;
	double td;
};

struct iidstruct {
	double mean;
	double variance;
};

static FILE *fp;
static struct azstruct  **aztable;
static struct iidstruct **iidtable;
double azimuth,f_old,iid2,iid;
int azindex;
static int NumAz,NumFreq1,NumFreq2,NumIid;

findgausstd(double f, double *t)
{
	double td;
	int findex;
	int onflag;

	onflag = [gaussfunction getazimuth:&azimuth];
	if (onflag==1)
	{
		azindex = azimuth * AZCONST;
		findex = f / FCONST;
		if (azindex > NumAz)
			{
			azindex = NumAz;
			printf("azindex too high for array.%d\n",NumAz);
			}
		if (findex > NumFreq1)
			{
			findex = NumFreq1;
			printf("findex too high at %d\n",NumFreq1);
			}
		td = aztable[azindex][findex].td;
		/*printf("azi=%d fi=%d td = %f\n",azindex,findex,td);*/
		*t = td;
	}
}

findmv(double f,double *m, double *v)
{
	double mm, vv;
	int findex,iidindex,sign;
	int azflag = 1;
	sign=1;
	
	findex = f / FCONST;
	iidindex = iid2* IIDCONST*azflag;
	if(iidindex<0) {
		iidindex=-iidindex;
		sign = -1;
	}
	if (iidindex > NumIid) {
		iidindex = NumIid;
		printf("iidindex too high at %d\n",NumIid);
	}
	if (findex > NumFreq2) {
		findex = NumFreq2-1;
		printf("findex too high for iid at %d\n",NumFreq2);
	}
	mm = iidtable[iidindex][findex].mean*azflag*sign;
	vv = iidtable[iidindex][findex].variance;
	*m = mm;
	*v = vv;
}

/* Actual gaussian pulse function */
gaussinit(double m, double v)
{
	int i;
	double x, v2;
	double *ptr = &(gauss_array[-DISP_TLOW]);

	v2=v*v;
	if ((-DISP_TLOW+DISP_TUP+1) > 1024)
		printf("gauss error.\n");
		
	for(i=DISP_TLOW;i<=DISP_TUP;i++) {
		x = i * DISP_TSTEP;
		ptr[i] = Exp(-(x-m) * (x-m) / (2 * v2)) / (sqrt(2*PI*v2));
	}
}

/**********************************************************************
	Experimental non-gaussian pulse function (Hanning window)
	v is defined to be the size, in msec, of the window.  We have
	to convert msec to samples (OS2 uses var. M/2)
 **********************************************************************/
kkkkgaussinit(double m, double v)
{
	int i;
	double x, tp, w;
	double *ptr = &(gauss_array[-DISP_TLOW]);
	
	if ((-DISP_TLOW+DISP_TUP+1) > 1024) printf("gauss error.\n");
	
	m /= (DISP_TSTEP);	// convert to samples
	v /= (DISP_TSTEP); // now v is like M/2 in OS2 equation
		
	for(i = DISP_TLOW; i <= DISP_TUP; i++) {
		if ( i < m - v || i > m + v)
			w = 0;
		else
			w = ( 1.-cos(2*PI*(double)(i - m - v)/(2*v)) )/2;
		
		ptr[i] = w;
	}
}


/*************************************************************************
	Gaussian pulse function with variance inversely proportional
	to the mean, so if we assume that the variance is one when mean
	is 5 ms, then we can determine the constant to linearly decrease the
	variance by the equation
		(v0 - 1) = m(.005 - 0)
	where v0 is the variance at x=0, and we calculate m, the slope.
	
	The "new" Gaussian pulse will then have variance = v0 - m*x.
	x is the mean of the pulse.
	Here, we assume for x=DESIRED_MEAN ms, the width is DESIRED_WIDTH.
  *************************************************************************/
#define DESIRED_WIDTH 1
#define DESIRED_MEAN 5
yyygaussinit(double m, double v)	// mean in ms, variance in ms, too
{
	int i;
	double x, v2, slope;
	double *ptr = &(gauss_array[-DISP_TLOW]);

	v2 = v*v;		// find variance, but this is screwed
	slope = (v2 - DESIRED_WIDTH)/(DESIRED_MEAN);
	v2 -= slope*m*(m/fabs(m));	// decrease variance by slope*current mean value
	
	if ((-DISP_TLOW+DISP_TUP+1) > 1024) printf("gauss error.\n");
	for(i=DISP_TLOW;i<=DISP_TUP;i++) {
		x = i * DISP_TSTEP;
		ptr[i] = Exp(-(x-m)* (x-m) / (2 * v2)) / (sqrt(2*PI*v2));
	}
}


double gaussmap(double x, double f, double t)
{
	double m,v,iid,td;
	/*static double f_old;*/
	int findex,iidindex,index,azflag;
	double *ptr = &(gauss_array[-DISP_TLOW]);
	
	findex = f / FCONST;
	azflag = 1;
	if (f_old != f) {
		if (azindex > NumAz) {
			azindex = NumAz;
			printf("azindex too high for array.%d\n",NumAz);
		}
		if (azindex < 0) { 
		    azflag = -1;
			azindex *=azflag;
		}
		if (findex > NumFreq1) {
			findex = NumFreq1-1;
			printf("findex too high at %d\n",NumFreq1);
		}
		iid = aztable[azindex][findex].iid*azflag;
		td = aztable[azindex][findex].td*azflag;
		printf("azi=%d fi=%d td=%f, iid=%f\n",azindex,findex,td,iid);
		iidindex = iid * IIDCONST*azflag;
		if (iidindex > NumIid) {
			iidindex = NumIid;
			printf("iidindex too high at %d\n",NumIid);
		}
		if (findex > NumFreq2) {
			findex = NumFreq2-1;
			printf("findex too high for iid at %d\n",NumFreq2);
		}
		m = iidtable[iidindex][findex].mean*azflag;
		v = iidtable[iidindex][findex].variance;
		mean = m;
		deviation = v;
		/*printf("mean=%f, var=%f\n",m,v);*/
		gaussinit(m,v);
		f_old = f;
	}
	index = (t / DISP_TSTEP);
	return x * ptr[index];
}

double gauss(double t)
{
	double m, v;
	int index;
	double *ptr = &(gauss_array[-DISP_TLOW]);
	
	m = mean;
	v = deviation;

	gaussinit(m,v);
	index = (t / DISP_TSTEP);
	return ptr[index];
}

double testmv(double x, double f, double t)
{
	double m,v,iid,td;
	/*static double f_old;*/
	int findex,index;
	double *ptr = &(gauss_array[-DISP_TLOW]);
	
	if (f_old!=f) {
		[gaussfunction getMean:&m];
		[gaussfunction getVariance:&v];
		mean = m;
		deviation = v;
		/* printf("\n mean=%f, var=%f\n",m,v); */
		gaussinit(m,v);
		f_old = f;
	}
	index = (t / DISP_TSTEP);
	return x * ptr[index];
}

double gaussmap2(double x, double f, double t)
{
	double m, v, iid2, td;
	/*static double f_old;*/
	int findex, iidindex, index, azflag;
	double*ptr = &(gauss_array[-DISP_TLOW]);
	findex = f / FCONST;
	azflag = 1;
	if (f_old != f) {
		if (findex > NumFreq1) {
			findex = NumFreq1-1;
			printf("findex too high at %d\n",NumFreq1);
		}
		iidindex = iid * IIDCONST*azflag;
		if (iidindex > NumIid) {
			iidindex = NumIid;
			printf("iidindex too high at %d\n",NumIid);
		}
		if (findex > NumFreq2) {
			findex = NumFreq2-1;
			printf("findex too high for iid at %d\n",NumFreq2);
		}
		m = iidtable[iidindex][findex].mean*azflag;
		v = iidtable[iidindex][findex].variance;
		/*printf("mean=%f, var=%f\n",m,v);*/
		gaussinit(m,v);
		f_old = f;
	}
	index = (t / DISP_TSTEP);
	return x * ptr[index];
}

double iidmap(double x, double f, double t)
{
	double m,v,y;
	/*static double f_old;*/
	int findex,iidindex,index,azflag,i,sign;
	double*ptr = &(gauss_array[-DISP_TLOW]);
		
	findex = f / FCONST;
	azflag = 1;
	sign = 1;
	if (f_old != f) {
		if (findex > NumFreq1) {
			findex = NumFreq1-1;
			printf("findex too high at %d\n",NumFreq1);
		}
		iidindex = iid2* IIDCONST*azflag;
		if(iidindex<0) {  
			iidindex = -iidindex;
			sign = -1;
		}
		if (iidindex > NumIid) {
			iidindex = NumIid;
			printf("iidindex too high at %d\n",NumIid);
		}
		if (findex > NumFreq2) {
			findex = NumFreq2-1;
			printf("findex too high for iid at %d\n",NumFreq2);
		}
		m = iidtable[iidindex][findex].mean*sign;
		v = iidtable[iidindex][findex].variance;
		mean = m;
		deviation = v;
		/*printf("mean=%f, var=%f\n",m,v);*/ 
		/*printf("\n iid = %f, mean = %f\n",iid2, mean);*/
		gaussinit(m,v);
		f_old = f;
	}
	index = (t / DISP_TSTEP);
	return x * ptr[index];
}


void loadaztable(void)
{
	double tmp1,tmp2;
	int a,f;
	char *azfilename=(char *) malloc(30);
	azfilename=getenv(PVMODEL_AZTABLE);
	
	if ((fp = fopen("../tables/az.tab", "r")) == NULL )	{
		if ((fp=fopen(azfilename, "r")) == NULL) {
			printf("Gauss.m: Can't find az.tab in ../tables or %s in $PVMODEL_AZTABLE paths\n", azfilename);
			return;
			}
		}
	else strcpy(azfilename, "../tables/az.tab");
		
	printf("loading az.tab in %s\n", azfilename);
	fscanf(fp,"%d:%d",&NumAz,&NumFreq1);
	aztable = (struct azstruct **) malloc(sizeof(struct azstruct *)
			* NumAz);
	for(a=0;a<NumAz;a++)
		{
		*(aztable+a) = (struct azstruct *)
			 malloc(sizeof(struct azstruct) * NumFreq1);
		for(f=0;f<NumFreq1;f++)
			{
			fscanf(fp,"%lf,%lf",&tmp1,&tmp2);
			aztable[a][f].iid = tmp1;
			aztable[a][f].td = tmp2;
			}
		}
	printf("Finished loading az table.\n");
	fclose(fp);
}

void loadiidtable(void)
{
	double tmp1,tmp2;
	int a,f,numfreq;
	char *iidfilename=getenv(PVMODEL_IIDTABLE);
	
	if ((fp = fopen("../tables/iid.tab", "r")) == NULL )	{
		if ((fp=fopen(iidfilename, "r")) == NULL) {
			printf("Gauss.m: Can't find iid.tab in ../tables or %s in $PVMODEL_IIDTABLE paths!\n", iidfilename);
			return;
			}
		}
	else strcpy(iidfilename, "../tables/iid.tab");
		
	printf("loading iid.tab in %s\n", iidfilename);
	fscanf(fp,"%d:%d",&NumIid,&NumFreq2);
	iidtable = (struct iidstruct **) malloc(sizeof(struct iidstruct *)
			 * NumIid);
	for(a=0;a<NumIid;a++) {
		*(iidtable+a) = (struct iidstruct *) 
			malloc(sizeof(struct iidstruct) * NumFreq2);
		for(f=0;f<NumFreq2;f++) {
			fscanf(fp,"%lf,%lf",&tmp1,&tmp2);
			iidtable[a][f].mean = tmp1;
			iidtable[a][f].variance = tmp2;
		}
	}
	fclose(fp);
	printf("Finished loading iid table.\n");
}

