/* Module that finds crosscorrelation function of Hafter experiment.
 *  i.e. noise gated with time delayed pulse functions.
 *
 *	(August 88)	TOR - Created.
 *	(8-24-88)	TOR - fixed active region bug.
 *	(8-26-88)	TOR - commented.
 *	1-5-89		TOR - changed flow to make it legible.
 *	1-7-89		TOR - added CCFout parameter.
 *	1-8-89		TOR - added crunch capability.
 *	1-16-89		TOR - added crunched mag,phase capability.
 * 	(5-30-89)	TOR - adapted for NeXT.
 */

#import <math.h>
#import <stdio.h>
#import <objc/objc.h>
#import "BiDis.h"
#import "Nerve.h"
#import "ccf.h"
#import <stdlib.h>

/* for spectrum, output phase? */
/* writes out numbers instead */
/* 0=none, 1=noise,2=gate,3=both */
/* 4 = crunch(sum CCF over freq) */
/* 5 = crunched mag/phase plot */
/* ms to plot in CCFout (+-) */

#define msout	2.5

extern id mytext;
extern double 	fixfreq(double f,double t),
				HmSqd(double f,double fm),
				active(double spl,double f),
				getflow(),
				getfhigh(),
				Sqrt(double x);
extern int 	isinregion(double f,double fm,double frac);
extern Regiontypes regiontype(void);
extern ccf *makeptau(ccf *lm);
extern 	fft(float *buf,int pts,int nump,int d),
		multfft(int pts,float *m1,float *m2,float *p),
		nrectify(double vart,double tf,double ttd,double varn,
			double *np,int tup,double tstep,double *op),
		lowpass(double *sig,int tup,double tstep),
		addtext(id obj,char *str);

/* returns pointer to freq array of bp noise. */
makenoise(double spl, double fl, double fh, float *ptr)
{
	double freq,n0,fstep = 1000.0/(NREALPTS*DISP_TSTEP);
	int f;

	n0 = pow(10.0,spl/20.0) * Sqrt(fstep * .5 / (fh-fl));
	for(f=0,freq = 0.0; f<NREALPTS; f+=2,freq += fstep) {
		if (freq <= fl || freq >= fh) {
			ptr[f] = ptr[f+1] = 0.0;
		} else {
			ptr[f] = n0;
			ptr[f+1] = 0.0;
		}
	}
	ptr[1] = 0.0;
}

/* produces undelayed gate function. */
makegate(double gf, double ln, float *ptr)	// Hz, msec, msec
{
	int n,tmp;
	double nxtstart, pstart = 1000.0/(gf * DISP_TSTEP);
	int plength = ln/DISP_TSTEP;

	nxtstart = 0.0;
	for(n=0;n<NREALPTS;n++) {
		ptr[n] = 0.0;
		if (n == (int) nxtstart) {
			nxtstart += pstart;
			tmp = 0;
			do {
				ptr[n++] = .5; /* fft */
			} while(n<NREALPTS && ++tmp < plength);
			n--;
		}
	}
}

/* returns crosscorellation function of ptr1,ptr2 */
makeCCF(float *ptr1, float *ptr2)
{
	double fact = 1.0/(NREALPTS);
	int n;
	for(n=3;n<NREALPTS;n+=2)		/* make sure N1 is even */
		ptr1[n] = -ptr1[n];
	multfft(NCOMPPTS,ptr1,ptr2,ptr1);
	fft(ptr1,NCOMPPTS,LOGNCOMPPTS,0);
	for(n=0;n<NREALPTS;n++)
		ptr1[n] *= fact;		/* fft and crosscor scaler */
}

/* Hm filters signals (spectral density functions) */
double hmCCF(double f, float *N, double *ptr)
{
	int n;
	float h[NREALPTS];
	double freq,fstep = 1000.0/(NREALPTS*DISP_TSTEP);

	for(n=2,freq=fstep;n<NREALPTS;n+=2,freq += fstep) {
		N[n] *= HmSqd(freq,f);
		N[n+1] *= HmSqd(freq,f);
/*		h[n] = HmSqd(freq,f);	for hm filter printout.
		h[n+1] = 0.0;*/
	}
/*	h[1] = HmSqd(freq,f);
	fft(h,NCOMPPTS,LOGNCOMPPTS,0);
	for(n=0;n<NREALPTS;n++) printf("%d 	%f\n",n,h[n]); */
	N[0] *= HmSqd(0.0,f);
	N[1] *= HmSqd(freq,f);
	fft(N,NCOMPPTS,LOGNCOMPPTS,0);
	/* note: I do not divide by NREALPTS because GDS took care of that
	   in the nrectify procedure somehow. */
	ptr[0] = (double) N[0];
	for(n=1;n<=DISP_TUP;n++) {
		ptr[n] = (double) N[n];
		ptr[-n] = (double) N[NREALPTS - n];
	}
}

/* note: N should be real freq response. This function finds the energy
   of an autocorrellation function. */
double getvar(double f, float *N)
{
	int n;
	double halfvar,freq,fstep = 1000.0/(NREALPTS*DISP_TSTEP);

	halfvar = 0.0;
	for(n=0,freq=0.0;n<NREALPTS;n+=2,freq+=fstep) {
		halfvar += N[n] * HmSqd(freq,f);
	}
	halfvar += N[1] * HmSqd(freq,f);
	return 2 * halfvar;
}

/* copies ptr2 into ptr1 with a shift of td ms right. */
shiftcopy(float *ptr2, float *ptr1, double td)
{
	int n, tn = td/DISP_TSTEP;

	for(n=0;n<tn;n++)
		ptr1[n] = ptr2[NREALPTS-tn+n];
	for(n=tn;n<NREALPTS;n++)
		ptr1[n] = ptr2[n-tn];
}


/* changes frequencies to multiples of incrementals. */
freq_init(double *f0, double *bw, double *fm, double *ts)
{
	double tmp;

	*f0 = fixfreq(*f0,DISP_TSTEP);
	tmp = 2000.0/(NREALPTS*DISP_TSTEP);
	*bw = (double)((int)(*bw/tmp+.5))*tmp;
	*fm = fixfreq(*fm,DISP_TSTEP);
	*ts = (double) ((int)(*ts/DISP_TSTEP+.5))* DISP_TSTEP;
}


/* this routine outputs a 2-d CCF numerically. */
writeCorr(float *X)
{
	int n,points;
	float maxval;
	char str[100];

	points = msout / DISP_TSTEP;
	maxval = fabs(X[0]);
	for(n=1;n<points;n++) {
		if (fabs(X[n]) > maxval) maxval = fabs(X[n]);
		if (fabs(X[NREALPTS-n]) > maxval) maxval = fabs(X[n]);
	}
	maxval /= 100.;
	sprintf(str,"0 %f\n",X[0]/maxval);
	addtext(mytext,str);
	for(n=1;n<points;n++) {
		sprintf(str,"%f %f\n",n*DISP_TSTEP,X[n]/maxval);
		addtext(mytext,str);
		sprintf(str,"%f %f\n",-n*DISP_TSTEP,X[NREALPTS-n]/maxval);
		addtext(mytext,str);
	}
}


/* returns the raw CCF of the stimulus. */
/* or, if CCFout is !0, writes crosscor function directly. */
getstimulus(float *Ncorr, float *Nauto, double noiseflow, double noisefhigh,
			double SPL, double pl, double fm, double ts, int CCFout)
{
	int n;
	float gate1[NREALPTS],gate2[NREALPTS],gate3[NREALPTS];

	makenoise(SPL,noiseflow,noisefhigh,Ncorr);	/* in freq domain */
	shiftcopy(Ncorr,Nauto,0.0);			/* just copy */
	makeCCF(Ncorr,Nauto);			/* Ncorr should be even */
	if (CCFout == 1) {
		writeCorr(Ncorr);
	}
	makegate(fm,pl,gate1);
	shiftcopy(gate1,gate2,ts);
	fft(gate1,NCOMPPTS,LOGNCOMPPTS,1);
	fft(gate2,NCOMPPTS,LOGNCOMPPTS,1);
	shiftcopy(gate1,gate3,0.0);		/* autocorr. gate. */
	makeCCF(gate3,gate1);			/* auto. */
	makeCCF(gate1,gate2);
	if (CCFout == 2) {
		writeCorr(gate1);
	}
	for(n=0;n<NREALPTS;n++) {
		Nauto[n]=Ncorr[n]*gate3[n];  /* Nauto is autocorr function */
		Ncorr[n] *= gate1[n];	/* Ncorr is crosscor function */
	}
	if (CCFout == 3) {
		writeCorr(Ncorr);
	}
}

/* routine makes ccf function to lowest active freq. */
calcCCF(ccf *Lm, float *Ncorr, float *Nauto, double minflow, double maxfhigh, double *dispflp, double *dispfhp, double f0)
{
	int startfreq,actFlag,n;
	double FRate = pow(10.0,DISP_FSTEP);
	double TmpArr[-DISP_TLOW+DISP_TUP+1],
			*TmpPtr = &TmpArr[-DISP_TLOW];
	double Varm,Frac,Frac1,Frac2,Frac0;
	float temp1[NREALPTS],temp2[NREALPTS];
	
	if (regiontype() == fixedband) {
		startfreq = (maxfhigh+minflow) / 2.;
	} else startfreq = log10(f0)/DISP_FSTEP;	/* center of noise band */
	*dispflp = *dispfhp = pow(FRate,(double)startfreq);
	actFlag = 1;
	while (actFlag > 0)	{	/* first loop till below active region */
		*dispflp = pow(FRate,(double)startfreq);
		for(n=0;n<NREALPTS;n++) {
			temp1[n] = Ncorr[n];
			temp2[n] = Nauto[n];
		}
		Varm = getvar(*dispflp,temp2);
		hmCCF(*dispflp,temp1,TmpPtr);
		nrectify(0.0, 1.0, 0.0,Varm, TmpPtr,
			 DISP_TUP, DISP_TSTEP, Lm->R[startfreq]);
		lowpass(Lm->R[startfreq], DISP_TUP, DISP_TSTEP);
		Frac = active(10.0*log10(Varm),*dispflp);
		if (isinregion(f0,f0,Frac) < 0.0) {		// single fiber
			actFlag = -1;
		}
		if (!isinregion(*dispflp,f0,Frac*Frac) 
		    || startfreq <= dtoi(minflow)) actFlag = 0;
		Frac2 = Frac*Frac;
		Frac0 = (1.-Frac)*(1.-Frac);
		Frac1 = 1. - Frac2 - Frac0;
		Frac2 *= DISP_NFIBERS_STEP;
		Frac0 = DISP_NFIBERS_STEP * (
				Frac1 *NERVE_SPONT_RATE*NERVE_ACTIVE_RATE +
				Frac0 *NERVE_SPONT_RATE*NERVE_SPONT_RATE);
		beg_t_loop(Lm)
			Lm->R[startfreq][t] = 
				Frac2 *Lm->R[startfreq][t] + Frac0;
		end_t_loop
		startfreq --;
	}
	startfreq = log10(*dispfhp)/DISP_FSTEP + 1;
	while (actFlag > -1) {	/* loop till above active region. */
		*dispfhp = pow(FRate,(double)startfreq);
		for(n=0;n<NREALPTS;n++) {
			temp1[n] = Ncorr[n];
			temp2[n] = Nauto[n];
		}
		Varm = getvar(*dispfhp,temp2);
		hmCCF(*dispfhp,temp1,TmpPtr);
		nrectify(0.0, 1.0, 0.0,Varm, TmpPtr,
			 DISP_TUP, DISP_TSTEP, Lm->R[startfreq]);
		lowpass(Lm->R[startfreq], DISP_TUP, DISP_TSTEP);
		Frac = active(10.0*log10(Varm),*dispfhp);
		if (!isinregion(*dispfhp,f0,Frac*Frac)
			|| startfreq >= dtoi(maxfhigh)) actFlag = -1;
		Frac2 = Frac*Frac;
		Frac0 = (1.-Frac)*(1.-Frac);
		Frac1 = 1. - Frac2 - Frac0;
		Frac2 *= DISP_NFIBERS_STEP;
		Frac0 = DISP_NFIBERS_STEP * (
				Frac1 *NERVE_SPONT_RATE*NERVE_ACTIVE_RATE +
				Frac0 *NERVE_SPONT_RATE*NERVE_SPONT_RATE);
		beg_t_loop(Lm)
			Lm->R[startfreq][t] =
				 Frac2 *Lm->R[startfreq][t] + Frac0;
		end_t_loop
		startfreq++;
	}
}


/* finds correct magnitude of CCF. Could eliminate all previous constant
  mult factors to increase speed */
checkmag(float *ptr, double spl)
{
	double fact,fstep = 1000.0/(NREALPTS*DISP_TSTEP);
	int n;

	fact = /*fstep * */ pow(10.0,spl/10.0) / (ptr[0] * NREALPTS);
	fact *= .5;	/* for fft */
	for(n=0;n<NREALPTS;n++)
		ptr[n] *= fact;
}


/* integrates over all frequencies. */
crunch(ccf *Lm, ccf *Rm)
{
	double sum,numfreqs;

	numfreqs = (double) (Rm->f_up - Rm->f_low) + 1.0;
	beg_t_loop(Rm)
		sum = 0.0;
		beg_f_loop(Rm)
			sum += Rm->R[f][t];
		end_f_loop
		Lm->R[Lm->f_low][t] = sum / numfreqs;
	end_t_loop
}


/* displays magnitude of CCF vs freq */
displaymag(float *N, int num)
{
	int n;
	double freq,fstep;
	char str[100];

	fstep = 1000.0/(num*DISP_TSTEP);
	for(n=2,freq= fstep;n<num-1;n+=4,freq += 2*fstep) {
		sprintf(str,"%f %f\n",freq,Sqrt(N[n]*N[n]+N[n+1]*N[n+1]));
		addtext(mytext,str);
	}
	sprintf(str,"0 %lf\n%f %f\n",N[0],freq,N[1]);
	addtext(mytext,str);
}


/* displays phase of CCF vs freq */
displayphase(float *N, int num)
{
	int n;
	double freq,fstep;
	char str[100];

	fstep = 1000.0/(num*DISP_TSTEP);
	for(n=2,freq= fstep;n<num-1;n+=4,freq += 2*fstep) {
		sprintf(str,"%f %f\n",freq,atan2(N[n+1],N[n]));
		addtext(mytext,str);
	}
	printf("0 0.0\n%f 0.0\n",freq);
	addtext(mytext,str);
}


/* main routine that returns CCF of noise and gate function */
ccf *hafter(double SPL, double f0, double fm, double bw,
			double pl, double ts, int CCFout)
{
	ccf *Lm,*Rm;
	float Ncorr[NREALPTS],Nauto[NREALPTS];
	float tmp[1024];
	double noiseflow,noisefhigh, FRate = pow(10.0, DISP_FSTEP);
	double fcm, DispFL, DispFH,maxfhigh,minflow;
	int n;
	char str[100];

	freq_init(&f0,&bw,&fm,&ts);
	noiseflow = f0 - bw/2.0;
	noisefhigh = f0 + bw/2.0;
	minflow = log10(getflow())/DISP_FSTEP;
	maxfhigh = log10(getfhigh())/DISP_FSTEP;
	Lm = newccf(minflow,maxfhigh,DISP_FSTEP,(double)DISP_TLOW,
		(double)DISP_TUP, DISP_TSTEP);
	getstimulus(Ncorr,Nauto,noiseflow,noisefhigh,SPL,pl,fm,ts,CCFout);
	checkmag(Ncorr,SPL);
	checkmag(Nauto,SPL);
/*	displaycorr(Ncorr); */
/* 	return Lm; */
	fft(Ncorr,NCOMPPTS,LOGNCOMPPTS,1);	/* need freq domain */
	fft(Nauto,NCOMPPTS,LOGNCOMPPTS,1);
/*	displaymag(Ncorr,NREALPTS); */
/* 	displayphase(Ncorr,NREALPTS); */
/*	return Lm;*/
	calcCCF(Lm,Ncorr,Nauto,minflow,maxfhigh,&DispFL,&DispFH,f0);
	Rm = newccf(log10(DispFL)/DISP_FSTEP,log10(DispFH)/DISP_FSTEP,
	DISP_FSTEP,(double)DISP_TLOW,(double)DISP_TUP,DISP_TSTEP);
	beg_f_loop(Rm)
		beg_t_loop(Rm)
			Rm->R[f][t] = Lm->R[f][t];
		end_t_loop
	end_f_loop
	freeccf(Lm);
	if (CCFout == 4) {
		Lm = newccf(0.0,0.0,DISP_FSTEP,(double)DISP_TLOW,
			(double)DISP_TUP,DISP_TSTEP);
		crunch(Lm,Rm); /* create single freq ccf funtions. */
		freeccf(Rm);
		return Lm; 	/* single freq */
	}
	if (CCFout == 5) {
		Lm = newccf(0.0,0.0,DISP_FSTEP,(double)DISP_TLOW,
			(double)DISP_TUP,DISP_TSTEP);
		crunch(Lm,Rm); /* create single freq ccf funtions. */
		freeccf(Rm);
		for(n= 0;n < DISP_TUP;n++)
			tmp[n]  = Lm->R[0][n];
		for(n= DISP_TUP;n < DISP_TUP-DISP_TLOW;n++)
			tmp[n] = Lm->R[0][n-(DISP_TUP-DISP_TLOW)];
		fft(tmp,DISP_TUP,9,1);
		displaymag(tmp,2*DISP_TUP);
		sprintf(str,";phase time:\n");
		addtext(mytext,str);
		displayphase(tmp,2*DISP_TUP);
		return Lm;
	} else
		return Rm;
}


/* original hafter lateralization.  Used to confirm Hafter's results. */
hafterlat(float *ptr, ccf *Lm)
{
	int n;
	int pts = DISP_TUP;
	double t1,t2;
	ccf *ptau;

	t1 = t2 = 0;
	for(n=1;n<=DISP_TUP;n++) {
		Lm->R[0][n] = ptr[n];
		Lm->R[0][-n] = ptr[NREALPTS-n];
	}
	Lm->R[0][0] = ptr[0];
	ptau = makeptau(Lm);
	ccfmul(Lm,ptau,Lm);
	for(n= -pts;n<=pts;n++) {
		t1 += Lm->R[0][n] * n;
		t2 += abs(Lm->R[0][n]);
		printf("%d	%f\n",n,Lm->R[0][n]);
	}
/*	t1 *= DISP_TSTEP/(2*pts+1); */
	t1 /= t2;
	printf("L = %f\n",t1);
}


spectrum(double SPL, double f0, double fm, double bw,
		 double pl, double ts, int phase)
{
	ccf *Lm,*Rm;
	float Ncorr[NREALPTS],Nauto[NREALPTS];
	double noiseflow,noisefhigh, FRate = pow(10.0, DISP_FSTEP);
	double fcm, DispFL, DispFH,maxfhigh,minflow,Frac,fstep;
	int n;
	char str[100];

	freq_init(&f0,&bw,&fm,&ts);
	noiseflow = f0 - bw/2.0;
	noisefhigh = f0 + bw/2.0;
	minflow = log10(getflow())/DISP_FSTEP;
	maxfhigh = log10(getfhigh())/DISP_FSTEP;
	Lm = newccf(minflow,maxfhigh,DISP_FSTEP,(double)DISP_TLOW,
		(double)DISP_TUP, DISP_TSTEP);
	getstimulus(Ncorr,Nauto,noiseflow,noisefhigh,SPL,pl,fm,ts,0);
	checkmag(Ncorr,SPL);
	checkmag(Nauto,SPL);
	fft(Ncorr,NCOMPPTS,LOGNCOMPPTS,1);	/* need freq domain */
	fft(Nauto,NCOMPPTS,LOGNCOMPPTS,1);

	fstep = 1000.0/(NREALPTS*DISP_TSTEP);
	for(n=2,Frac= fstep;n<NREALPTS-1;n+=8,Frac += 4*fstep) {
		if (phase == 0)
		sprintf(str,"%f %lf\n",
			Frac,Sqrt(Ncorr[n]*Ncorr[n]+Ncorr[n+1]*Ncorr[n+1]));
		else
			sprintf(str,"%f %lf\n",Frac,atan2(Ncorr[n+1],Ncorr[n]));
		addtext(mytext,str);
	}
	if (phase == 0)
		sprintf(str,"0 %lf\n",Ncorr[0]);
	else
		sprintf(str,"0 %lf\n",atan2(0,Ncorr[0]));
	addtext(mytext,str);
}


/* displays N vs time */
displaycorr(float *N)
{
	int n;
	char str[100];

	for(n=1; n<=40; n++) {
		sprintf(str,"%d	%f\n%d %f\n",n,N[n]-n,N[NREALPTS-n]);
		addtext(mytext,str);
	}
	sprintf(str,"0	%f\n",N[0]);
	addtext(mytext,str);
}


