/*
*	I0.c -- I0()  zero-order modified Bessel function of the first kind.
*  HISTORY
*	(5-23-87)	GDS - Added I(m,z).
*	(5-25-89)	TOR - adapted for NeXT.
*
*/
#import <stdio.h>
#import <math.h>

#define SUMTOL		0.0001	/* Determines stopping condition for sums. */


double I0(register double x)
{
	int k;
	register double i0 = 1.0, i0k = 1.0;

	x = x*x/4.;

	for (k = 1; k < 100 && i0k/i0 > SUMTOL; k++) {
		i0k *= (x/(k *k));
		i0 += i0k;
	}
	return i0;
}

double I(int m, register double x)		/* "Kind" of Mod. Bessel function */
{
	int k;
	register double termmul, term = 1.0, sum = 1.0;

	if (m < 0) {
		fprintf(stderr,"WARNING from I(): changing m=%d to m=0.\n",m);
		m = 0;
	}

	x /= 2.0;
	termmul = x * x;

	for (k = 1; k < 100 && term/sum > SUMTOL; k++) {
		term *= (termmul / (k*(k+m)));
		sum += term;
	}

	for(k = m; k > 0;)
		sum *= (x / k--);

	return sum;
}

