/*
*	lowpass.c
*  HISTORY
*	2-2-87		GDS - Released and tested on tones and DC signals.
*	4-24-87		GDS - Fixed bug in transfer from signal array to
*			input array.  (Did not appear to effect results).
*			Enhanced TEST code. Noted that for some reason the
*			routines do not function properly when NREALPTS is
*			2048 (but it works for NREALPTS equals 512, 1024).
*			(4-27-87 discovered that error was caused by TABLESIZE
*			in subfft.c  The cos and sin tables limit the size of
*			FFT that can be computed.) 
*	5-31-87		GDS - Added call to G_HasChanged().
*	7-14-88		TOR - fixed bug by dividing fft by 2 and invfft by
*			NREALPTS. G should not be multiplied by two.
*	(5-24-89)	TOR - adapted for NeXT.
*/

#import <stdio.h>
#import "BiDis.h"
#import "Nerve.h"

#define WARNING	1	/* Defined to enable certain validity checks */

#define	FFT		1	/* Direction flags */
#define INVFFT	0	/* for fft() routine. */

extern double	GSqd(double f);
extern int	G_HasChanged();
extern fft(), multfft();

static float	filter[NREALPTS], input[NREALPTS];
static double	curtstep = -1.0, /* msec */	curfstep;	/* Hz */

void lowpassinit(double newtstep)	/* ms */
{
	register int f;
	double freq;

	if ( newtstep == curtstep && !G_HasChanged() )
		return;
	curtstep = newtstep;
	curfstep =  (1000. / (NREALPTS * curtstep));	/* Hz */

	for (f = 0, freq = 0.0; f < NREALPTS; f += 2, freq += curfstep) {
		filter[f] = GSqd(freq);
		filter[f+1] = 0.0;
	}
	filter[1] = GSqd(freq);
}

/* 7-15-88	TOR - had to divide by 2 and NREALPTS for FFT and INVFFT
		respectively. Did not change above testing routine. */
void lowpass(double *signal, int tup, double tstep)		/* tstep in ms */
{
	register int t;

	lowpassinit(tstep);

	input[0] = signal[0];
	
	for(t = 1; t <= tup; t++) {
		input[t] = signal[t] / 2;				/* /2 for fft */
		input[NREALPTS - t] = signal[-t] / 2;
	}
	for(t = tup + 1; t < NREALPTS - tup;)
		input[t++] = 0.0;

	fft(input, NCOMPPTS, LOGNCOMPPTS, FFT);
	multfft(NCOMPPTS, input, filter, input);
	fft(input, NCOMPPTS, LOGNCOMPPTS, INVFFT);

	signal[0] = input[0] / NREALPTS;
	
	for(t = 1; t <= tup; t++) {
			signal[t] = input[t] / NREALPTS;		/*4096 because of INVFFT*/
			signal[-t] = input[NREALPTS - t] / NREALPTS;
	}
}

