/* These are definitions from ccfsup.c */
#import <stdio.h>
#import "ccf.h"
#import <stdlib.h>
#import "Nerve.h"

extern double DISP_NFIBERS_STEP;
extern Fraction *activeTable;

static ccf *straight_ccf;

double pow(double, double);

extern float elapsed_real_time(int);	// for timing routines
extern float elapsed_user_time(int);	// for timing routines
extern float elapsed_sys_time(int);	// for timing routines
extern	float my_r_time;		// for timing routines
extern	float my_u_time;
extern	float my_s_time;


/* straightness routine constants */
#define MAX_ORDER 22	// max order we can handle
extern Fraction *activeTable;
static double sum = 0;		/* holds straightness result */
static int skip, fringe;
extern int strb4_order;
static int slc_f, slc_t;	// let str_loop access ccf_str's slc_f index		   

/***************************************************************************
	"New straightness" routine; found later to be equivalent to
	"old straightness".  This routine could be faster if we didn't use
	recursion, but instead assumed some large MAX_ORDER and just
	looped for straightness and broke out.
 ***************************************************************************/
void str_loop(int depth)
{
	static int j[MAX_ORDER];
	static double P = 1;
	register int k, pf;
	
	for (j[depth] = 0; j[depth] <= 2; j[depth]++) {
		if (depth >= strb4_order) {		// do product
			for (k = 1, pf = slc_f - straight_ccf->f_low, P = 1.0;
				 k <= strb4_order; k++, pf += skip) {
				if (j[k] == 0) {
					P *= SPONT_ACTIVE_PROD  * activeTable[pf].n0;
				} else if (j[k] == 1) {
					P *= SINGLY_ACTIVE_PROD * activeTable[pf].n1;
				} else if (j[k] == 2) {
					P *= straight_ccf->R[pf + straight_ccf->f_low][slc_t]
					     * activeTable[pf].n2;
				}
			}
			sum += P;
		} else {
			str_loop(depth+1);
		}
	}
}
		
/**************************************************************************
	Straightness routine that corrects the false assumption that the
	fibers in each group of straightness are doubly-active.
		
	df  - the bandwidth of frequency counter
	strfstep - the frequency step of the straightness algorithm
 **************************************************************************/
ccf *ccf_str(ccf *cr, int df)	/* df is the order */
{
	extern double DISP_FSTEP;	/* The value of fstep */
	extern double strb4_fstep;
	extern int strb4_flag;
	ccf *cstr;
	register int endpointt, endpointf;		
	
/**************************************************************************
	Do no straightness case.  Note that the CCF elements that are DA
	should have the value DOUBLY_ACTIVE_PROD in it from the CCF routines
 **************************************************************************/
   	if (strb4_flag == 0 || df <= 1) {
		beg_t_loop(cr)
			beg_f_loop(cr)
			// This loop could be faster, but doesn't need to be.
				cr->R[f][t] *= activeTable[f - cr->f_low].n2;
				cr->R[f][t] +=
					SINGLY_ACTIVE_PROD * activeTable[f - cr->f_low].n1 +
					SPONT_ACTIVE_PROD  * activeTable[f - cr->f_low].n0;
				cr->R[f][t] *= DISP_NFIBERS_STEP;
			end_f_loop
		end_t_loop
	
		return cr;
	}
	
/***********************************************************************
	Here, skip is how many freq elements to skip over, i.e. the number of
	elements at the beginning and end of the matrix is
	fringe = skip*(half of the order-1)/2
 ***********************************************************************/
	skip = (int)((strb4_fstep / DISP_FSTEP) + 0.5);
	if (skip == 0)
		skip = 1;
	skip *= strb4_flag;
	fringe = skip * (df - 1)/2;
	
/***********************************************************************
	Create a *ccf to hold the result of straightness.  The straightness
	routine looks at the fringes, but isn't ever centered in the fringe
	area, so the *cstr will be exactly like *cr, except that it'll be
	smaller by fringe at the edges, and overall by 2*fringe.
	
	Also make a global (to this file) copy of the CCF before straightness
	so that we can use it in the loop above.
 ***********************************************************************/
	cstr = newccf(cr->f_low + fringe, cr->f_up - fringe, cr->fstep,
				  cr->t_low, cr->t_up, cr->tstep);
	if (cstr == (ccf *)0) {
	
		freeccf(cr);
		return cstr;	// Bad straightness matrix allocation
	}
	straight_ccf = ccfcopy(cr);
	
	endpointt = cr->t_up;
	endpointf = cr->f_up - 2 * fringe;	// assuming odd orders
	
	for (slc_t = cr->t_low; slc_t <= endpointt; slc_t++) {
		for (slc_f = cr->f_low; slc_f <= endpointf; slc_f++) {
			/* Calculate straightness using fstep*skip increments */
			/* Perform derived equation: */
			/* slc == "second level (coincidence) counter" */
			
			sum = 0;
			str_loop(1);	// Calculate straightness
			
/*********************************************************************
	Put sum into str matrix -- the way we're doing the f loop above forces
	us to properly offset the str matrix
 *********************************************************************/
			cstr->R[slc_f + fringe][slc_t] = DISP_NFIBERS_STEP * sum;
		}
	}
	

	freeccf(cr);
	return cstr;	/* Send back straightness matrix */
}

