/*
*	noise.c
*  HISTORY
*	(3-15-87)	GDS - altered call to rectify to account for new
*				rectifier model.
*	(3-20-87)	GDS - added OneFiber flag.
*	(4-6-87)	GDS - Modified to use REGION module and made FStep
*				a variable. Changed fc to f0 and td to ts.
*	(5-8-87)	GDS - Added noiseactive() routine for call by user.
*				Added DebugOn switch.
*	(5-25-89)	TOR - adapted for NeXT.
*/

#import <math.h>
#import <stdlib.h>
#import "BiDis.h"
#import "Nerve.h"
#import "ccf.h"
#import <appkit/nextstd.h>

extern Fraction *activeTable;

extern double	NoiseVarHmdB(double spl, double fl, double fh, double fm);
extern Fraction	NoiseActive(double SPL, double IID,
							double fl, double fh, double fm);
extern double	NoiseCCF(double spl, double fl, double fh,
		 		double td, double ps, double fm, double *outp,
				int tup, double tstep);
extern 	nrectify(double vart, double tf, double ttd, double varn,
				 double *np, int tup, double tstep, double *op),
		lowpass(double *sig, int tup, double tstep),
		findgausstd(double freq, double *td);
extern int isinregion(double fc, double f0, Fraction thisfraction);
extern execerror(char *s, char *t);

ccf *noise(double SPL, double IID, double f0, double ts, double ps, double bw)
{
	ccf *Lm;
	Fraction fraction;
	double StimFL = f0 - bw/2., StimFH = f0 + bw/2.;
	double FRate = pow(10.0, DISP_FSTEP);
	double fm, DispFL = -1.0, DispFH = -1.0, Varm, fl, fu;
	double	TmpArr[-DISP_TLOW + DISP_TUP + 1], *TmpPtr = &TmpArr[-DISP_TLOW];
	int fringe;
	extern double strb4_fstep;
	extern int strb4_order;
	extern int strb4_flag;
		
	/* Set up CCF data structure */
	fraction = NoiseActive(SPL, IID, StimFL, StimFH, fm);
	if (isinregion(fm, f0, fraction) < 0.0)
		DispFL = DispFH = f0;
	else
		for (fm = 20.; fm <= 20000.; fm *= FRate) {
			fraction = NoiseActive(SPL, IID, StimFL, StimFH, fm);
#ifdef DEBUG
printf("fm = %f\tfm/FRate = %f\tfrac2 = %.2f frac1 = %.2f\n", fm, fm/FRate, fraction.n2, fraction.n1);
#endif*/
			if (DispFL < 0.0) {
				if (isinregion(fm, f0, fraction)) {
					DispFL = fm;
#ifdef DEBUG
printf("*\n");
#endif*/
				}		
			} else if (!isinregion(fm, f0, fraction)) {
				DispFH = fm / FRate;
#ifdef DEBUG
printf("**\n");
#endif
				break;
			}
		}
#ifdef DEBUG
printf("DispFL=%f	DispFH=%f\n", DispFL, DispFH);
#endif
	
	if (DispFL < 0 || DispFH < 0)
		return(Lm=0);
		
/**************************************************************************
	The next allocation is the basis for Tao's straightness weighting.
	It uses a window centered at a certain f and extending out on both
	sides of f.  The width of the window depends on two variables:
	
	(1) the skip factor, which is straightness_fstep/fstep, or how many freqs
	to skip over when multiplying adjacent amplitudes when computing
	straightness, and
	(2) the order, which is how many amplitudes to look at when computing
	straightness.  Assume that order (==df) is odd so that the window is
	symmetric, e.g., if order=7, the window will extend 3*skip frequencies
	on either side of the "center frequency."
 **************************************************************************/
	 
	fringe = (int) ((strb4_fstep/DISP_FSTEP) + 0.5);
	if (fringe == 0)
		fringe = 1;		/* Round (int)ed result up to 1 */
	/* Allocate extra freqs for straightness iff flag on */
	fringe *= strb4_flag*(strb4_order-1)/2;
	printf("noise: strb4_order=%d, fringe=%d\n", strb4_order, fringe);
	
	fl = log10(DispFL)/DISP_FSTEP - fringe;
	fu = log10(DispFH)/DISP_FSTEP + fringe;
	Lm = newccf(fl, fu, DISP_FSTEP,
				(double)DISP_TLOW, (double)DISP_TUP, DISP_TSTEP);

	/********************** Create fractions table *********************/
	activeTable = (Fraction *) calloc(fu - fl + 1, sizeof(Fraction));
	printf("Creating activeTable from %.1f to %.1f, 0 to %.1f\n", fl, fu, fu-fl+1);

/****************************************************************************
 	Fill in the crosscorrelation function, but scale by DISP_NFIBERS_STEP
	in straightness routine, not here
 ****************************************************************************/	
	beg_freq_loop(Lm)
		/* Put fractions into table */
		activeTable[f - Lm->f_low]
			= NoiseActive(SPL, IID, StimFL, StimFH, freq);
			
		findgausstd(freq, &ts);		/* Acct. for azimuth, if necessary */
		
		/* Compute unrectified CCF funtion */
		Varm = NoiseCCF(SPL, StimFL, StimFH, ts, ps, freq, TmpPtr,
						DISP_TUP, DISP_TSTEP);
		
		/* Rectify it */
		nrectify(0.0, 1.0, 0.0, Varm, TmpPtr, DISP_TUP, DISP_TSTEP, Lm->R[f]);
		
		/* Lowpass filter it */
		lowpass(Lm->R[f], DISP_TUP, DISP_TSTEP);

		/* Manual rectification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
		beg_t_loop(Lm)
			if (Lm->R[f][t] < 0.0)
				Lm->R[f][t] = 0.0;
		end_t_loop */
		
	end_freq_loop
	
	return Lm;
}


/* If you want to see the differences between finding output power
	using (1) Exact, analytical expressions for noise output PSD
	and (2) "Integration" of area under output PSD, stick the next lines
	a few lines up in noise() */
/* printf("freq=%.1f:  aT=%.1f # Varm=%.1f\n",
freq, NoiseVarHmdB(SPL, StimFL, StimFH, freq), 10.*log10(Varm)); */
				
