/*
*	ptauLogFD.c -- K*log((|f|+l)/(|f|+h)) char. function (ptauFT())
*			with frequency dependent "peak-clipper".
*  HISTORY
*			WARNING: ptauFT() and ptauFTdf() do not account for
*			"Mu" parameter (i.e., they are only valid when
*			Mu = 0.0).
*	(4-7-87)	GDS - Copied from ptauLog.c.  Added frequency
*			dependence to Mu.
*	(5-3-87)	GDS - Added frequency dependence to l and h.
*	(5-10-87)	GDS - Added normalization at each freq.
*	(6-23-87)	GDS - Added FreqThresh parameter.  Previous
*			predictions are the same as FreqThresh big.
*	(7-14-88)	TOR - Added function makeptau.  Ptau is now an
*			array and calculated before lat once.
*	(5-24-89)	TOR - adapted for NeXT.
*       (3-28-90)       XXU - added new ptau/Li(0) function.
*       
*/
#include <stdio.h>
#include <math.h>
#include "ccf.h"
#include "BiDis.h"
#import "Nerve.h"

#define ROOT2PI	2.506628275

extern double sinc(double x);
int		order2 = 5;

double mean=0.0,
       deviation= 1.778;

double	lF = 0.1,
		lP = -1.15, 
		hF = 3000., 
		hP = 0.0, 
		MuF = 0.2e-3, 
		MuP = 0.0,
		Fthresh = 1200.;

static double K = -1.0, L = -1., H = -1., mn = 0.0, l = -1., h = -1.;

double ptau(double t)		/* t in msec */
{
	double tmp;
	t /= 1000.;

	if (fabs(t) <= mn) t = mn;
	if ((t = fabs(t)) <= 1.e-23) t = 2.*PI*K*(h-l);
	else t = K*(exp(-2.*PI*l*t) - exp(-2.*PI*h*t))/t;

	errno = 0;
	return t;
}


double ptauFT(double f)		/* f in Hz */
{
	f = K*log((f*f+H)/(f*f+L));
	errno = 0;
	return f;
}

double ptauFTdf(double f)		/* f in Hz */
{
	return -2.*K*(H-L)*f / ((f*f+H)*(f*f+L));
}

ptauinit(double f)		/* f in Hz */
{
	double area = 0.0, tau;
	errno = 0;

	if (f > Fthresh)
		f = Fthresh;

	mn = MuF/pow(f, MuP);
	l = lF/pow(f, lP);
	h = hF/pow(f, hP);

	K = .5/Log(h/l);
	L = l*l;
	H = h*h;
	if (fabs(mn) > 0.0) {
		for (tau = 0.0; tau < DISP_TUP*DISP_TSTEP; tau += DISP_TSTEP)
			 area += ptau(tau);
		area *= 2.*DISP_TSTEP/1000.;
		K /= area;
	}

	return 1;
}

static double prevf= -1.0;

double ptaumap(double x, double f, double t)	/* f in Hz, t in msec */
{
	if (f != prevf) {
		ptauinit(f);
		prevf = f;
	}
	return x *ptau(t);
}

ccf *makeptau(ccf *L)
{
	ccf *P;

	P = newccf(L->f_low,L->f_up,L->fstep,L->t_low,L->t_up,L->tstep);
	beg_freq_loop(P)
	ptauinit(freq);
	beg_time_loop(P)
		P->R[f][t] = ptau(time);
	end_time_loop
	end_freq_loop
	return P;
}

double newptau(double t, double m, double v)	/* msec */
{
	double tmp1,tmp2;
	double order3;
	order3=order2+0.5;
/*	tmp2 = sqrt(2*PI)*v;*/
/*	tmp2=pow(tmp2,order2);*/
	if(t>=0.5)
	tmp1 = exp(0.25*order2/(2.*v*v));
	else
	tmp1 = exp(t*t*order2/(2.*v*v));
	t /= 1000.; 
        if (fabs(t) <= mn)
	 t = mn;
	 if ((t = fabs(t)) <= 1.e-23) t = 2.*PI*K*(h-l);
	 else
	 t = tmp1*K*(exp(-2.*PI*l*t) - exp(-2.*PI*h*t))/t;
	errno = 0;
	return t;
}



