/*
*	PVLat.c
* HISTORY
*	(3-15-87)	GDS - Added DebugOn (and ass.) variable.
*	(3-25-87)	GDS - Added Litau().
*	(4-6-87)	GDS - Eliminated Litau().
*	(5-10-87)	GDS - Added ccfsave() calls. (Does not affect
*			predictions, so did not change LatVars[] string.)
*	(6-30-87)	GDS - Added Kaiser() window option.
*	(5-25-89)	TOR - adapted for NeXT.
*        (2-14-90)	XXU - added straightness-weighted mode.
*        (3-20-90)	XXU - added intensity weighting function to straightness  weighting model
*/

#import <stdio.h>
#import <math.h>
#import "ccf.h"

#define DbgFreq		500.0

static double WindowGain=0.0;

extern int errno,order2;
extern double	iid2, mean, deviation;
extern double	ptaumap(double x,double f,double t),
		testmv(double x, double f, double t),
		Kaiser(int a,int b,double c),
		newptau(double t, double m, double v),
		findmv(double f, double m, double v);
extern int	ptauinit(double f);
extern int	peakpick_on, raatgever_on;

/***************************************************************************
	Apply frequency salience (Raatgever) weighting and ptau(f, c) function.
	Then sum across frequency to form a 1-D array and find centroid of the
	1-D array.  This function is called by lateralizer.m.
 ***************************************************************************/
double lateralize(ccf *c)
{
	double sal, freq_salience(), ptau();
	double sum, salsum;
	ccf *Lt;
	int freqdep = ptauinit(500.);
	int DbgFindex;

	errno = 0;

	if (!raatgever_on) {	// No Raatgever weighting
		beg_freq_loop(c)
			if (freqdep)
				ptauinit(freq);
			beg_time_loop(c)
				c->R[f][t] *= ptau(time);
			end_time_loop
		end_freq_loop
	} else {				// Raatgever weighting
		beg_freq_loop(c)
			if (freqdep)
				ptauinit(freq);
			beg_time_loop(c)
				c->R[f][t] *= freq_salience(1.0, freq)*ptau(time);
			end_time_loop
		end_freq_loop
	}
	
	/* cofa */
	/****** For each int delay, sum over all frequencies => 1-D array *******/
	Lt = integ_df(c);
	
	/* Kaiser window routine no longer used here:
	   Now that WindowGain is zero, Kaiser() returns 1, so just skip this: */
	// WindowGain = fabs(WindowGain);
	//		beg_t_loop(Lt)
	//			Lt->R[Lt->f_low][t] *= Kaiser(t, Lt->t_up, WindowGain);
	//		end_t_loop
			
	sal = ctrd_peak(Lt, Lt->f_low);
	printf("*** peak-picking finds maximum at %.4f \n", sal);
	sal = ctrd_dt(Lt, Lt->f_low);		// Get centroid of 1-D array
	printf("*** centroid finds average at %.4f \n", sal);
	
	if (peakpick_on) {
		sal = ctrd_peak(Lt, Lt->f_low);		// Get peak of 1-D CCF array
		if (sal == -999999) {
			printf("Peaks of CCF too similar in magnitude -- finding centroid instead.\n");
			sal = ctrd_dt(Lt, Lt->f_low);
		}
	} else
		sal = ctrd_dt(Lt, Lt->f_low);		// Get centroid of 1-D array
	
	freeccf(Lt);
	return sal;
	/* only */



/********************* This is an old routine *************************/
/* avg of centroid
	sum = 0.0;
	salsum = 0.;
	beg_freq_loop(c)
		sal = freq_salience(1.0, freq);
		sum += (ctrd_dt(c, f) * sal);
		salsum += sal;
	end_freq_loop
	return sum/salsum;
*/
}



/*************************** Experimental Ptau ******************************/
double lateralize_c(ccf *c)    
{
	double sal, freq_salience(), newptau();
	double sum, salsum,m,v;
	ccf *Lt, *Lt2;
	int freqdep = ptauinit(500.);
	int DbgFindex;

	errno = 0;

	beg_freq_loop(c)	
/*		sal = freq_salience(1.0, freq);*/
                sal = 1.0;
		if (freqdep) ptauinit(freq);
		beg_time_loop(c)
			c->R[f][t] *= sal*newptau(time,mean,deviation);
		end_time_loop
	end_freq_loop
/* cofa */
	Lt = integ_df(c);
/*	c=ccf_nmstr(c,10);  */
/*	Lt2 = minteg_df(c); */
	WindowGain = fabs(WindowGain);
	beg_t_loop(Lt)
		Lt->R[Lt->f_low][t] *= Kaiser(t, Lt->t_up, WindowGain);
/*		Lt->R[Lt->f_low][t] *= Lt2->R[Lt2->f_low][t]; */
	end_t_loop
	sal = ctrd_dt(Lt, Lt->f_low);
	freeccf(Lt);
	return sal;
/* only */

}


