/*
*	RaatNoise.c
*	(7-10-87)	GDS - Copied from noise.c.
*	(6-5-89)	TOR - adapted for NeXT.
*/

#import <math.h>
#import <stdlib.h>
#import <stdio.h>
#import "BiDis.h"
#import "Nerve.h"
#import "ccf.h"

#define INVFFT		0

extern int	errno;
extern Fraction *activeTable;

extern double	NoiseVarHmdB(double spl, double fl, double fh, double fm);
extern Fraction RaatNoiseActive(double SPL, double IID, double fm);
extern Fraction NoiseActive(double SPL, double IID,
							double fl, double fh, double fm);
extern double	HmSqd(double f, double fm), Sqrt(double x);
extern int isinregion(double fc, double f0, Fraction thisfraction);
extern 	nrectify(double vart, double tf, double ttd, double varn,
				 double *np, int tup, double tstep, double *op),
		lowpass(double *sig, int tup, double tstep),
		fft(float *b, int p, int q, int d);
extern execerror(char *s, char *t);

static double	MagNar, MagWide;
static float	input[NREALPTS];

double Nmag(double f, double Wfl, double Wfu, double Nfl, double Nfu)
{
	if (f >= Nfl && f <= Nfu)
		return  MagNar;
	if (f >= Wfl && f <= Wfu)
		return MagWide;
	return 0.0;
}

double Nphase(double f, double Nts, double Wts, double Nfl, double Nfu)
{
	if (f >= Nfl && f <= Nfu)
		return -PI * f * Nts / 500.0;
	return -PI * f * Wts / 500.0;
}

/*$$$$$$$$$$$$$$$$$$$$$$$$ Haven't changed yet $$$$$$$$$$$$$$$$$$$$$$$$$$*/
double NoiseRrl(cf, tup, tstep, Nts, Wts, Wfl, Wfu, Nfl, Nfu, outp)
	double cf;		/* Characteristic Freq. (Hz) */
	double	tstep;		/* Time step (msec) */
	int	tup;		/* Valid range for outp[t] is -tup<= t<= tup*/
	double Nts, Wts, Wfl, Wfu, Nfl, Nfu;	/* Raat. noise parameters */
	double	*outp;		/* Array for results */
{
	register int f;
	double freq, mag, phase;
	double fstep = 1000.0 / (NREALPTS*tstep);	/* Hz */
	double halfvar = 0.0;

	/* Fill filter fft with freq. response */
	for (f = 0, freq = 0.0; f < NREALPTS;
	     f+= 2, freq += fstep) {
		phase = Nphase(freq, Nts, Wts, Nfl, Nfu);
		mag = 0.5 * Nmag(freq, Wfl, Wfu, Nfl, Nfu) * HmSqd(freq, cf) * fstep;
		halfvar += mag;
		input[f]   = mag * cos(phase);
		input[f+1] = mag * sin(phase);
	}
	input[1] = 0.;
	errno = 0;	/* clear any range errors */
	/* don't /NREALPTS because multfft not used. */
	fft(input, NCOMPPTS, LOGNCOMPPTS, INVFFT);

	outp[0] = input[0];
	for (f = 1; f <= tup; f++) {
		outp[f]  = input[f];
		outp[-f] = input[NREALPTS-f];
	}

	return 2.0 * halfvar;
}

ccf *RaatNoise(double SPL, double IID, double  DLnw, double  Nfl, double  Nfu,
			   double  Wfl, double  Wfu, double  Nts, double  Wts)
{
	ccf *Lm;
	Fraction fraction;
	double FRate = pow(10.0, DISP_FSTEP);
	double fm, DispFL = -1.0, DispFH = -1.0, Varm, fl, fu, f0;
	double	TmpArr[-DISP_TLOW + DISP_TUP + 1], *TmpPtr = &TmpArr[-DISP_TLOW];
	int fringe;
	extern double strb4_fstep;
	extern int strb4_order;
	extern int strb4_flag;

	MagWide = pow(10.0, SPL/10.0) / (Wfu - Wfl);
	MagNar = MagWide * pow(10.0, DLnw/10.0);
	f0 = Sqrt(Nfu * Nfl);
	
	/* Set up CCF data structure */
	fraction = NoiseActive(SPL, IID, Wfl, Wfu, fm);
	if (isinregion(fm, f0, fraction) < 0.0)
		DispFL = DispFH = f0;
	else
		for (fm = 20.; fm <= 20000.; fm *= FRate) {
/*	$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	Strictly speaking, this routine to calculate the fraction of active
	fibers will only work if MagWide & MagNar are of equal amplitude 
	(DLnw = 0).  In that case, the stimulus is equivalent to a bandpass
	noise of BW (Wfu - Wfl).  Some parts of the noise are phase-shifted,
	but that doesn't matter for the output (filtered) power.  In practice,
	the results are accurate to about +/- 1 dB.
	$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$  */
			fraction = NoiseActive(SPL, IID, Wfl, Wfu, fm);
			if (DispFL < 0.0) {
				if (isinregion(fm, f0, fraction)) {
					DispFL = fm;
				}		
			} else if (!isinregion(fm, f0, fraction)) {
				DispFH = fm / FRate;
				break;
			}
		}
	
	if (DispFL < 0 || DispFH < 0)
		return(Lm = 0);

/**************************************************************************
	The next allocation is the basis for Tao's straightness weighting.
	It uses a window centered at a certain f and extending out on both
	sides of f.  The width of the window depends on two variables:
	
	(1) the skip factor, which is straightness_fstep/fstep, or how many freqs
	to skip over when multiplying adjacent amplitudes when computing
	straightness, and
	(2) the order, which is how many amplitudes to look at when computing
	straightness.  Assume that order (==df) is odd so that the window is
	symmetric, e.g., if order=7, the window will extend 3*skip frequencies
	on either side of the "center frequency."
 **************************************************************************/
	 
	fringe = (int) ((strb4_fstep/DISP_FSTEP) + 0.5);
	if (fringe == 0)
		fringe = 1;		/* Round (int)ed result up to 1 */
	/* Allocate extra freqs for straightness iff flag on */
	fringe *= strb4_flag*(strb4_order-1)/2;
	printf("noise: strb4_order=%d, fringe=%d\n", strb4_order, fringe);
	
	fl = log10(DispFL)/DISP_FSTEP - fringe;
	fu = log10(DispFH)/DISP_FSTEP + fringe;
	Lm = newccf(fl, fu, DISP_FSTEP,
				(double)DISP_TLOW, (double)DISP_TUP, DISP_TSTEP);
				
	/********************** Create fractions table *********************/
	activeTable = (Fraction *) calloc(fu - fl + 1, sizeof(Fraction));
	printf("Creating activeTable from %.1f to %.1f, 0 to %.1f\n", fl, fu, fu - fl + 1);

/****************************************************************************
 	Fill in the crosscorrelation function, but scale by DISP_NFIBERS_STEP
	in straightness routine, not here
 ****************************************************************************/	
	beg_freq_loop(Lm)
		/* Don't put fractions into table yet */
		/* Unlike noise() routine, we get power of output noise by
		   "integrating" power spectrum, so we need to call the Raat CCF
		   routine to get the noise variance.  Another way to get the
		   power would have been to find the noise output from each of
		   the three bands of noise.  IF the amplitudes of the three
		   bands are equal, we would only need to call the routine once,
		   like noise() does. */
		   
/*$$$$$$$$$$$$$$$$$$$$$ No azimuth here!  See noise.c $$$$$$$$$$$$$$$$$$$$$*/

		/* Compute unrectified CCF funtion */
		Varm = NoiseRrl(freq, DISP_TUP, DISP_TSTEP,
						Nts, Wts, Wfl, Wfu, Nfl, Nfu, TmpPtr);
		
		/* Put fractions into table */
		activeTable[f - Lm->f_low]
			= RaatNoiseActive(10.0*log10(Varm), IID, freq);
			
		/* Rectify it */
		nrectify(0.0, 1.0, 0.0, Varm, TmpPtr, DISP_TUP, DISP_TSTEP, Lm->R[f]);

		/* Lowpass filter it */
		lowpass(Lm->R[f], DISP_TUP, DISP_TSTEP);

		/* Manual rectification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */
		beg_t_loop(Lm)
			if (Lm->R[f][t] < 0.0)
				Lm->R[f][t] = 0.0;
		end_t_loop
		
	end_freq_loop
   
	return Lm;
}
