/*
*	region.c -- Routines to determine range of fibers to be considered
*		by central processor.
*  HISTORY
*	(4-6-87)	GDS - Created.
*	(8-25-88)	TOR - wrote getflow, getfhigh, regiontype.
*				(no sweat.)
*	(5-24-89)	TOR - adapted for NeXT.
*/

#import <math.h>
#import "ccf.h"

// See ccf.h for regiontypes enum type
/* TYPE: 1 => OneFiber; 2 => ActiveThresh; 3 => Fixed; 4 => CriticalBand */

#define CBFrac	1.0
extern double actThresh;
extern double Flow, Fup;
extern Regiontypes thisRegion;

extern void execerror(char *s, char *t);

double fiber_bw(double f)	/* Critical Bandwidth of fiber with freq. f (Hz)*/
{
	if (f < 500.0)
		return 100.0;
	if (f <= 2000.0)
		return 0.1*f + 50.0;
	return 0.344*f - 438.0;
}


/*********************************************************************
	fc -- characteristic Freq. of fiber (Hz)
	f0 -- center Freq. of stimulus (Hz)
	Frac2 -- fraction of "doubly-active" fibers
	
	The region is a global variable, so it's kinda inefficient to
	call this routine from the stimulus modules to calculate the
	region.  However, it doesn't take very, very much processor time.
 *********************************************************************/
int isinregion(double fc, double f0, Fraction thisfraction)
{
	double bw;
	
	switch(thisRegion) {
	case singlefiber:	/* One fiber */
		return -1;		/* Implies use only one Char. Freq. */
		break;
		
	case doublyactive:	/* CFs with at least one active fiber in the pair */
		if (thisfraction.n2 > actThresh) {
			return 1;
		} else {
			return 0;
		}
		break;
			
	case fixedband: 	/* Fixed band */
		if (Flow <= fc && fc <= Fup)
			return 1;
		else
			return 0;
		break;
		
	case criticalband:	/* Critical region */
		bw = CBFrac * fiber_bw(f0) / 2.;
		if (f0 - bw <= fc && fc <= f0 + bw)
			return 1;
		else
			return 0;
		break;
		
	default:
		execerror("Bad choice of regions -- stop", (char *)0);
		break;
	}
}



/*********************************************************************
	############## This is just a hack to fix after vacation! #############
	fc -- characteristic Freq. of fiber (Hz)
	f0 -- center Freq. of stimulus (Hz)
	Frac2 -- fraction of "doubly-active" fibers
	
	The region is a global variable, so it's kinda inefficient to
	call this routine from the stimulus modules to calculate the
	region.  However, it doesn't take very, very much processor time.
 *********************************************************************/
 // last arg is normally a struct Fraction
int Xisinregion(double fc, double f0, double thisfraction)
{
	double bw;
	
	switch(thisRegion) {
	case singlefiber:	/* One Fiber */
		return -1;		/* Implies use only one Char. Freq. */
		break;
		
	case doublyactive:	/* activeThresh */
		if (thisfraction > actThresh)
			return 1;
		else
			return 0;
		break;
			
	case fixedband: 	/* fixed band */
		puts("fixed region!");
		if (Flow <= fc && fc <= Fup)
			return 1;
		else
			return 0;
		break;
		
	case criticalband:	/* fraction of critical region */
		bw = CBFrac * fiber_bw(f0) / 2.;
		if (f0 - bw <= fc && fc <= f0 + bw)
			return 1;
		else
			return 0;
		break;
		
	default:
		execerror("Bad choice of regions -- stop", (char *)0);
		break;
	}
}



/* simply returns Flower. */
double getflow(void)
{
	return Flow;
}

/* simply returns Fupper */
double getfhigh(void)
{
	return Fup;
}

/* returns region type */
Regiontypes regiontype(void)	// fix routines that call this!
{
	return (int) thisRegion;
}
