/*
 * SUBFFT - contains fft and routines it uses to compute real fast Fourier
 *     transforms.
 *********************************************
 * Created by faa (see /usr/faa/exp/float/fft4.c)
 *
 * 6/28/84 - modified by leb:
 *    removed prbuf
 *    changed BITREV
 *    rearranged procedures, renamed some varaibles
 * 4-24-87	GDS - modified multfft() routine so that result array
 *		could be the same as one of the input arrays. (This routine
 *		was created by GDS sometime in 1986.)
 * 4-27-87	GDS - added error check so that FFTs of size greater than
 *		TABLESIZE cause an error message.  Also made TABLESIZE 4096
 *		instead of 1024 and changed the table names to cosftable[] and
 *		sinftable[] instead of cosf1024[] and sinf1024[].
 * 6-29-88	TOR - modified multfft() routine so that it wouldn't use
 *		 a scalefactor.
 *		when using fft() divide func by 2 before, after invfft,
 *		divide by NREALPTS.
 * 5-24-89	TOR - adapted for NeXT.
 * 7-8-89  	TOR - removed cos, sin tables.  Added trigtable.c
 */

#import <stdio.h>
#import <math.h>
#import "BiDis.h"
#import "Nerve.h"

#define ISODD(n) (1 & n)
extern float 	cosftable[],
	   			sinftable[];
#define TABLESIZE NREALPTS	/* This is the maximum size of the FFT */

struct complex {
	float real, imag;
};

unravel (buf, points, direct)
     struct complex *buf;
     int points,direct;
{
    register struct complex
        *a, *c;
    register float
        wr, wi, apcr, amcr, apci, amci;
    int
        wd, i, theta;
    float
        re, im;

    a = &buf[1];
    c = &buf[points - 1];
    wd = TABLESIZE / points;
    theta = wd;

    re = buf->real;
    if (direct) {
	buf->real = 2 * (re + buf->imag);
	buf->imag = 2 * (re - buf->imag);
    }
    else {
	buf->real = (re + buf->imag);
	buf->imag = (re - buf->imag);
    }

    for (i = 1; i <= points >> 1; i++) {
	apcr = a->real + c->real;
	apci = a->imag + c->imag;
	amcr = a->real - c->real;
	amci = a->imag - c->imag;

	if (direct) {
	    wr = cosftable[theta];
            wi = -sinftable[theta];
	}
	else {
	    wr = -cosftable[theta];    
            wi = -sinftable[theta];
	}

	theta += wd;
	re = (wr * apci) + (wi * amcr);
	im = (wi * apci) - (wr * amcr);

	a->real = apcr + re;
	a->imag = im + amci;
	c->real = apcr - re;
	c->imag = im - amci;

	c--;
        a++;
    }
}

brv (i, nbits)
     int i,nbits;
{
    register int
	arg, bit, highbitmask, lowbitmask,
	result = 0;

    arg = i;
    lowbitmask = 1;
    highbitmask = 1 << (nbits - 1);
    for (bit = nbits - 1; bit >= 0; bit -= 2) {
	result |= (arg >> bit) & lowbitmask;
	result |= (arg << bit) & highbitmask;
	lowbitmask <<= 1;
	highbitmask >>= 1;
    }
    return (result);
}



bitrev (buf, points, log2p)
     struct complex *buf;
     int points,log2p;
{
    register int
        i, bri;
    register short
        *BRCptr;
    struct complex
        temp;
    static short
        BRcache[TABLESIZE],
 	BRcacheN = 0; 

    if (points == BRcacheN) {
	BRCptr = BRcache;
	for (i = 0; i < points; i++, BRCptr++) {
	    if (*BRCptr > i) {
		temp = buf[i];
		buf[i] = buf[*BRCptr];
		buf[*BRCptr] = temp;
	    }
	}
    }
    else {
	if (points <= TABLESIZE) {
 	    BRcacheN = points;     
	    BRCptr = BRcache;
	}
	for (i = 0; i < points; i++) {
	    bri = brv (i, log2p);
	    if (points < TABLESIZE)
	        *BRCptr++ = bri;
	    if (bri > i) {
		temp = buf[i];
		buf[i] = buf[bri];
		buf[bri] = temp;
	    }
	}
    }
}

fft2 (buf, pairs)
     struct complex *buf;
     int pairs;
{
    register int
	cntr;
    register struct complex
	*a, *c;
    register float
	ar, ai;

    a = buf;
    c = &buf[1];

    for (cntr = 0; cntr < pairs; cntr++) {
	ar = a->real;
	ai = a->imag;
        a->real = ar + c->real;
	a->imag = ai + c->imag;
	c->real = ar - c->real;
	c->imag = ai - c->imag;
	a += 2; c += 2;
    }
}


fft4 (buf, passes, mdel, wd, quads, points, direct)
     struct complex *buf;
     int mdel,wd,quads,points,direct;
{
    register struct complex
	*a, *b, *c, *d;
    register int
	cntr, pass, index, theta, memincr;
    float
	wr1, wi1, wr2, wi2, wr3, wi3,
	apcr, apci, amcr, amci,
	bpdr, bpdi, jbmdr, jbmdi,
	br, cr, dr;

    a = buf;
    c = &buf[mdel];
    b = &buf[mdel * 2];
    d = &buf[mdel * 3];

    theta = 0;
    memincr = mdel << 2;

    for (pass = 0; pass < passes; pass++) {
	if (pass) {
	    wr1 = cosftable[theta];
	    wr2 = cosftable[theta * 2];
            wr3 = cosftable[theta * 3];

	    if (direct) {
		wi1 = -sinftable[theta];
		wi2 = -sinftable[theta * 2];
		wi3 = -sinftable[theta * 3];
	    }
	    else {
		wi1 = sinftable[theta];
		wi2 = sinftable[theta * 2];
		wi3 = sinftable[theta * 3];
	    }
	}
        for (cntr = 0; cntr < quads; cntr++) {
	    if (pass) {
		br = b->real;
		b->real = (br * wr1) - (b->imag * wi1);
		b->imag = (br * wi1) + (b->imag * wr1);

		cr = c->real;
		c->real = (cr * wr2) - (c->imag * wi2);
		c->imag = (cr * wi2) + (c->imag * wr2);

		dr = d->real;
		d->real = (dr * wr3) - (d->imag * wi3);
		d->imag = (dr * wi3) + (d->imag * wr3);
	    }

	    apcr = a->real + c->real;
	    apci = a->imag + c->imag;
	    amcr = a->real - c->real;
	    amci = a->imag - c->imag;
	    bpdr = b->real + d->real;
	    bpdi = b->imag + d->imag;
	   jbmdr = d->imag - b->imag;
	   jbmdi = b->real - d->real;

	    if (!direct) {
		jbmdr = (-jbmdr);
		jbmdi = (-jbmdi);
	    }
	    a->real = apcr + bpdr;
	    a->imag = apci + bpdi;
	    c->real = amcr - jbmdr;
	    c->imag = amci - jbmdi;
	    b->real = apcr - bpdr;
	    b->imag = apci - bpdi;
	    d->real = amcr + jbmdr;
	    d->imag = amci + jbmdi;

	    a += memincr;
	    b += memincr;
	    c += memincr;
	    d += memincr;
	}
	theta += wd;
	a = &buf[pass+1];
	c = (struct complex *) (a + mdel);
	b = (struct complex *) (c + mdel);
	d = (struct complex *) (b + mdel);
    }
}

fft(buf, points, log2p, direct)
     struct complex *buf;
     int points,log2p,direct;
{
    register int
        index, quads, i, wd,
        passes = 1,
        mdel = 1;

    wd = TABLESIZE >> 1;

    if (wd < points)
	/* execerror("Too many points for the FFT routine.", (char *)0) */;

    quads = points >> 2;

    /* ravel if a reverse fft */
    if (!direct) 
	unravel (buf, points, direct);

    bitrev (buf, points, log2p);

    if (ISODD (log2p)) {
	fft2 (buf, points >> 1);
 	wd >>= 1;
	mdel <<= 1;
	passes <<= 1;
	quads >>= 1;
    }

    while (quads) {
	fft4 (buf, passes, mdel, wd, quads, points, direct);
	passes <<= 2;
	mdel <<= 2;
	wd >>= 2;
	quads >>= 2;
    }

    /* unravel if a forward fft */
    if (direct)
	unravel (buf, points, direct);
}


/* just multiply ffts */
multfft (npts, m1, m2, p)
	register int		npts;
	register struct complex	*m1, *m2, *p;
{
	/* Each FFT must be divided by 2 to correct it, and the */
	/* INVFFT must be divided by (2 *npts) to correct it.   Thus,     */
	/* the total scale factor is (8 *npts) */
	/* TOR - yes, but we will do that ourselves... */

	float rtmp;

	/* Special packing of first and last points (only real parts) */
	/* First point (no complex) */
	p->real = m1->real * m2->real;
	/* Last point (no complex) */
	p->imag = m1->imag * m2->imag;

	while (--npts > 0) {
		rtmp = (++m1)->real * (++m2)->real;
		rtmp = (rtmp - m1->imag * m2->imag);
		(++p)->imag = (m1->real*m2->imag + m1->imag*m2->real);
		p->real = rtmp;
	}
}
