/* TIME
 *  Functions to compute CPU (user & sys) and REAL time elapsed.
 *    Elapsed_???_time returns the number of milliseconds elapsed since
 *    last (update = 1).
 *    If update = 0, just peek and return how much time has elapsed.
 *    If update = 1, actually change the from time for next call.   
 *
 * Kai-Fu Lee
 * 6/86 Created
 */

#include <sys/types.h>
#include <sys/times.h>
#include <sys/timeb.h>	/* For real time */

#define	HZ	60	/* 60 clock ticks per second */


float elapsed_user_time (update)
int update;
{
  float  ret;
  static struct tms ubefore, uafter;

  times (&uafter);
  ret = (double) (uafter.tms_utime - ubefore.tms_utime) / HZ;
  if (update)
    times (&ubefore);
  return (ret);
}


float elapsed_wall_time (update)
int update;
{
  float  ret;
  static struct timeb ubefore, uafter;

  ftime (&uafter);
  ret = (float) (uafter.time + uafter.millitm/1000.0 -
		 ubefore.time - ubefore.millitm/1000.0);
  if (update)
    ftime (&ubefore);
  return (ret);
}

float elapsed_sys_time (update)
int update;
{
  float  ret;
  static struct tms sbefore, safter;

  times (&safter);
  ret = (double) (safter.tms_stime - sbefore.tms_stime) / HZ;
  if (update)
    times (&sbefore);
  return (ret);
}

elapsed_real_time (update)
int   update;
{
  register  ret;
  static struct timeb rbefore,
                      rafter;

  ftime (&rafter);
  ret = 1000 * (rafter.time - rbefore.time) +
	(rafter.millitm - rbefore.millitm);
  if (update)
    ftime (&rbefore);
  return (ret);
}
