/*
*	tone.c
*  HISTORY
*	(3-21-87)	GDS - Altered tone() to use ToneELfcm().
*			Also changed fc to f0 and td to ts.
*	(4-6-87)	GDS - Modified to use REGION module. And made
*				FStep a variable.
*	(5-8-87)	GDS - added toneactive() routine for call by user.
*			Added DebugOn option.
*	(5-24-89)	TOR - adapted for NeXT.
*/

#import <stdio.h>
#import <stdlib.h>
#import <math.h>
#import "BiDis.h"
#import "Nerve.h"
#import "ccf.h"

extern Fraction	ToneActive(double SPL, double IID, double f, double fm);
extern double	threshold(double f);
extern int		isinregion(double, double, Fraction),
				ToneELfcm(double SPL, double f0, double ts,
						  double fcm, double *outp, int tup, double tstep);
extern void		execerror(char *s, char *t);

extern Fraction *activeTable;

ccf *tone(double SPL, double IID, double fc, double ts)
{
	ccf	*Lm;
	Fraction Frac;
	double fracDA, FRate, fm = 0;
	double DispFL = -1.0, DispFH = -1.0;
	double *signal, *TmpPtr;
	double SigArray[-DISP_TLOW + DISP_TUP + 1];
	double TmpArray[-DISP_TLOW + DISP_TUP + 1];
	double fl, fu;
	// char str[512];
	int fringe;
	extern double strb4_fstep;
	extern int strb4_order;
	extern int strb4_flag;

	FRate  = pow(10.0, DISP_FSTEP);
 	signal = &SigArray[-DISP_TLOW];
	TmpPtr = &TmpArray[-DISP_TLOW];
	
/************************* Set up CCF data structure ***********************/

/***************************************************************************
	The next allocation is the basis for Tao's straightness weighting.
	It uses a window centered at a certain f and extending out on both
	sides of f.  The width of the window depends on two variables:
	
	(1) the skip factor, which is straightness_fstep/fstep, or how many freqs
	to skip over when multiplying adjacent amplitudes when computing
	straightness, and
	(2) the order, which is how many amplitudes to look at when computing
	straightness.  Assume that order (==df) is odd so that the window is
	symmetric, e.g., if order=7, the window will extend 3*skip frequencies
	on either side of the "center frequency."
 ***************************************************************************/
	fringe = (int) ((strb4_fstep/DISP_FSTEP) + 0.5);
	if (fringe == 0)
		fringe = 1;		/* Round (int)ed result up to 1 */
	fringe *= strb4_flag*(strb4_order-1)/2;	/* Allocate extra freqs for straightness iff flag on */
	printf("tone: strb4_order=%d, fringe=%d\n", strb4_order, fringe);
		

/**************************************************************************
	Find frequency region for the stimulus.  This is a circular task:
	
	1.  We need to store the active fractions into the table.
	2.  We need the freq. bounds to allocate memory for the table.
	3.	To find the bounds, we need to run the same routine calculates
	the active fractions in (1.) above.
	
	Let's go ahead and calculate the active fractions twice, and later
	we'll see how much time is saved by only doing that once.  (In that
	case, we'd have to preassume a table size.)
 *************************************************************************/
	/* Be careful with orders of arguments, esp. fc, fm! */
	Frac = ToneActive(SPL, IID, fc, fm);
	if (isinregion(fm, fc, Frac) < 0)
		DispFL = DispFH = fc;
	else for (fm = 20.; fm <= 20000.; fm *= FRate) {
		Frac = ToneActive(SPL, IID, fc, fm);
#ifdef DEBUG
printf("fm=%.2f\tfm/FRate =%.2f  frac2=%.2f frac1=%.2f frac0=%.2f\n", fm, fm/FRate, Frac.n2, Frac.n1, Frac.n0);
#endif
		if (DispFL < 0.0) {
			if (isinregion(fm, fc, Frac)) {
				DispFL = fm;
#ifdef DEBUG
printf("*\n");
#endif
			}
		} else if (!isinregion(fm, fc, Frac)) {
			DispFH = fm/FRate;
#ifdef DEBUG
printf("**\n");
#endif
			break;
		}
	}
#ifdef DEBUG
printf("DispFL=%f	DispFH=%f\n", DispFL, DispFH);
#endif
	
	/* tone parameters out of bounds ==> return null for toneCCF.m to fix;
	   don't execerror to a fatal error! */
	if (DispFL <= 0 || DispFH <= 0)
		return((ccf *)Lm = 0);
		
	fl = log10(DispFL)/DISP_FSTEP - fringe;
	fu = log10(DispFH)/DISP_FSTEP + fringe;
	Lm = newccf(fl, fu, DISP_FSTEP,
				(double)DISP_TLOW, (double)DISP_TUP, DISP_TSTEP);

	/********************** Create fractions table *********************/
	activeTable = (Fraction *) calloc(fu - fl + 1, sizeof(Fraction));
	printf("Creating activeTable from %.1f to %.1f, 0 to %.1f\n", fl, fu, fu-fl+1);
	
	/* Compute crosscorrelation function for doubly-active fibers */
	ToneELfcm(threshold(fc) + 50., fc, ts, fc, signal,
			  DISP_TUP,	DISP_TSTEP);

puts("Starting tone CCF");
/****************************************************************************
 	Fill in the crosscorrelation function for the doubly-active fibers.
	Scale by DISP_NFIBERS_STEP in straightness routine, not here.
 ****************************************************************************/	
beg_freq_loop(Lm)
		/* Get fractions of doubly-/singly-active & spontaneous fiber pairs */
		activeTable[f - Lm->f_low] = ToneActive(SPL, IID, fc, freq);
		fracDA = (activeTable[f - Lm->f_low]).n2;
				
/****************************************************************************
	If any of the fibers are doubly-active, copy the pre-calculated DA CCF
	(signal) into the CCF array at this frequency.  The straightness routine
	will scale the number of coincidences according to the fraction of
	doubly-active fibers.
	
	If there are no doubly-active fibers at a CF, we don't care about the
	number of coincidences; i.e. we will calculate Lm_2(f, t)*n2, but n2=0.
	(See straightness routine.)  We could just leave garbage here, but it
	feels nicer to zero them out just in case.
 ****************************************************************************/
		if (fracDA > 0.0) {
			beg_t_loop(Lm)
				Lm->R[f][t] = signal[t];
			end_t_loop
		} else {
			beg_t_loop(Lm)
				Lm->R[f][t] = 0;
			end_t_loop
			continue;
		}
		
	end_freq_loop
	
	return Lm;
}
